//+------------------------------------------------------------------------------------+
#define     _CODE_      "RB"
#define     _NAME_      "RenkoBasic"
#define     _VERSION_   "1.013"
#define     _BUILD_     "2016-04-09 02h44"
#define     _TITLE_     _NAME_+" (v"+_VERSION_+") - MetaTrader MQL4+ Expert Advisor"
#property   version     _VERSION_
#property   copyright   "Copyright 2016, Fernando M. I. Carreiro, All rights reserved"
#property   link        "mailto:fmi@carreiro.co.pt?subject="+_NAME_+".v"+_VERSION_
#property   description _NAME_" v"_VERSION_
#property   description "MetaTrader MQL4+ Expert Advisor"
#property   description "A Basic Renko Trend Following Strategy"
#property   strict
//+------------------------------------------------------------------------------------+
// Using default TAB size of 3 ( but inserting SPACE characters instead of TAB )
//+------------------------------------------------------------------------------------+

//+------------------------------------------------------------------------------------+
// Includes and Library References
//+------------------------------------------------------------------------------------+

#include <stdlib.mqh>
#include <stderror.mqh>


//+------------------------------------------------------------------------------------+
// Special Macro Definitions & Enumerations
//+------------------------------------------------------------------------------------+

#define _TGVNAMETAG_ string   // To define Terminal Global Variable Tags/Names

enum
   ENUM_TRADING_SIGNAL        // Define Trading Signal States
      {
            _UP_LONG_      =  +1,   // Long, Up or Buy
            _SIDE_OUT_     =   0,   // Sideways, Out, All Trades or None
            _DOWN_SHORT_   =  -1    // Short, Down or Sell
      };
      
enum
   ENUM_STOPSIZE_TYPE         // Define the Stop/Step Size Type
      {
            _FIXED_PIPS_      =  0, // Fixed: Number of Pips
            _RATIO_ATR_       =  1, // Variable: Ratio of ATR
            _RATIO_BRICK_     =  2  // Variable: Ratio of Brick Size
      };

enum
   ENUM_TRAILING_TYPE         // Define Trading Signal States
      {
            _NO_TRAILING_     =  0, // No Trailing Stop
            _NORMAL_TRAILING_ =  1, // Always Apply Trailing Stop 
            _PROFIT_TRAILING_ =  2  // Apply Trailing only After it is in Profit Zone
      };
      

//--- Title Identification
input string
   strTitle                   = _TITLE_,           // Expert Advisor Title
   strBuild                   = _BUILD_;           // Version Build Timestamp

//--- Expert Advisor Settings
input string
   strPrimarySettings
      = "--- Expert Advisor Settings ---";   // Expert Advisor Settings

   //---- Define Renko Parameters and Variables
   input string
      strRenkoSettings
         = "--- Renko Settings ---"; // Renko Settings
   extern double
      dblBrickSizePips           = 25,       // Size of Renko Brick (Pips)
      dblReversalBrickRatio      = 2;        // Ratio of Bricks for Reversal
   _TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
      nameBrickSizePips          = "BrickSizePips (double)",
      nameReversalBrickRatio     = "ReversalBrickRatio (double)";
   int
      intRenkoUpCount            = 0,
      intRenkoDownCount          = 0,
      intRenkoUpGainCount        = 0,
      intRenkoDownGainCount      = 0,
      intRenkoBrickSizePoints,               // Size of Renko Brick (Points)
      intRenkoReversalPoints,                // Size of Reversal (Points)
      intRenkoBasePricePoints,               // Base Price for Renko Brick (Points)
      intRenkoUpperLimitPoints,              // Upper Limit for Renko Brick (Points)
      intRenkoLowerLimitPoints;              // Lower Limit for Renko Brick (Points)
   double
      dblRenkoBrickSize;                     // Size of Renko Brick (Delta Price)
   ENUM_TRADING_SIGNAL
      enumSignalRenkoTrend;                  // Signal for Basic Renko Trend


   //---- Define Stop/Step Size Parameters and Variables
   input string
      strStopsSettings
         = "--- Stops Size Settings ---";    // Stops Settings
   extern ENUM_TIMEFRAMES
      enumATRTimeframe           = PERIOD_CURRENT; // ATR Timeframe
   extern int
      intATRPeriod               = 14;       // ATR Period
   extern double
      dblATRFactor               = 1.0,      // ATR Factor to use for Size Calculations
      dblRiskStopSize            = -1,       // Size of Stop for Risk Calculation
      dblStopLossSize            = 0.7,      // Size of Stop-Loss
      dblTrailStopSize           = 2,        // Size of Trailing-Stop
      dblTrailStepSize           = 1,        // Size of Trailing-Step
      dblBreakEvenSize           = 2,        // Size of Stop for Break-Even
      dblTakeProfitSize          = 10,       // Size of Take-Profit
      dblStealthGapSize          = 0;        // Size of Stealth Stops Gap
   extern ENUM_STOPSIZE_TYPE
      enumRiskStopType           = _RATIO_BRICK_,  // Type of Stop for Risk Calculation
      enumStopLossType           = _RATIO_BRICK_,  // Type of Stop-Loss
      enumTrailStopType          = _RATIO_BRICK_,  // Type of Trailing-Stop
      enumTrailStepType          = _RATIO_BRICK_,  // Type of Trailing-Step
      enumBreakEvenType          = _RATIO_BRICK_,  // Type of Stop for Break-Even
      enumTakeProfitType         = _RATIO_BRICK_,  // Type of Take-Profit
      enumStealthGapType         = _RATIO_BRICK_;  // Type of Stealth Stops Gap
   extern ENUM_TRAILING_TYPE
      enumTrailingType           = _PROFIT_TRAILING_; // Type of Trailing-Stop to Apply
   extern bool
      boolCloseOnReverseSignal   = false,    // Close Opposite Orders on Reverse Signal
      boolCheckBarExtreme        = false,    // Check Bar Extreme for Stop Loss
      boolCompensateSpread       = true;     // Compensate for Spread in Stops 
   _TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
      nameATRTimeframe           = "ATRTimeframe (enum)",
      nameATRPeriod              = "ATRPeriod (int)",
      nameATRFactor              = "ATRFactor (double)",
      nameRiskStopSize           = "RiskStopSize (double)",
      nameStopLossSize           = "StopLossSize (double)",
      nameTrailStopSize          = "TrailStopSize (double)",
      nameTrailStepSize          = "TrailStepSize (double)",
      nameBreakEvenSize          = "BreakEvenSize (double)",
      nameTakeProfitSize         = "TakeProfitSize (double)",
      nameStealthGapSize         = "StealthGapSize (double)",
      nameRiskStopType           = "RiskStopType (enum)",
      nameStopLossType           = "StopLossType (enum)",
      nameTrailStopType          = "TrailStopType (enum)",
      nameTrailStepType          = "TrailStepType (enum)",
      nameBreakEvenType          = "BreakEvenType (enum)",
      nameTakeProfitType         = "TakeProfitType (enum)",
      nameStealthGapType         = "StealthGapType (enum)",
      nameTrailingType           = "TrailingType (enum)",
      nameCloseOnReverseSignal   = "CloseOnReverseSignal (bool)",
      nameCheckBarExtreme        = "CheckBarExtreme (bool)",
      nameCompensateSpread       = "CompensateSpread (bool)";
   bool
      boolTrailing,                          // Apply Trailing Stop
      boolTrailingAfterProfit;               // Apply Trailing Stop only After Profit
   int
      intRiskStopDeltaPoints,                // Risk-Stop Delta (Points)
      intStopLossDeltaPoints,                // Stop-Loss Delta (Points)
      intTrailStopDeltaPoints,               // Trailing-Stop Delta (Points)
      intTrailStepDeltaPoints,               // Trailing-Step Delta (Points)
      intBreakEvenDeltaPoints,               // Break-Even Delta (Points)
      intTakeProfitDeltaPoints,              // Take-Profit Delta (Points)
      intStealthGapDeltaPoints;              // Stealth Stop Gap Delta (Points)
   double
      dblATRBase;                            // Base value for Stops based on ATR

   //---- Define Order Conditions Parameters and Variables
   input string
      strConditionsSettings
         = "--- Order Conditions Settings ---"; // Renko Settings
   extern ENUM_TRADING_SIGNAL
      enumTradePositions         = _SIDE_OUT_;  // Trading Positions Filter
   extern int
      intRangeDepthPeriod        = 12;       // Period for Range Depth Filter
   extern bool
      boolResetOnRangeFilter     = false,    // Reset Signal for Range Depth Filter
      boolFadeSignal             = false,    // Fade instead of Trend following signal
      boolMultipleOrders         = true,     // Allow Multiple Directional Orders
      boolRetriggerSignal        = false,    // Retrigger Signal when hitting original S/L
      boolOrderStacking          = true;     // Stack Orders when S/L is in Profit
   _TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
      nameTradePositions         = "TradePositions (enum)",
      nameRangeDepthPeriod       = "RangeDepthPeriod (enum)",
      nameResetOnRangeFilter     = "ResetOnRangeFilter (bool)",
      nameFadeSignal             = "FadeSignal (bool)",
      nameMultipleOrders         = "MultipleOrders (bool)",
      nameRetriggerSignal        = "RetriggerSignal (bool)",
      nameOrderStacking          = "OrderStacking (bool)";
   bool
      boolRangeDepthFilter;                  // Is Range Depth Filter enabled?
   int
      intRangeDepthHighPoints,               // High Price of Range Depth Filter (Points)
      intRangeDepthLowPoints;                // Low Price of Range Depth Filter (Points)


   //---- Define Time Filter Parameters and Variables
   input string
      strTimeFilterSettings
         = "--- Time Filter Settings ---"; // Time Filter Settings
   extern bool
      boolTimeFilterEnable    = true,        // Enable Time Filtering
      boolResetOnTimeFilter   = false;       // Reset Signal when Time Range is Invalid!
   extern datetime
      dtWeekStartExample      = D'2015.11.23 00:05:00',  // Start Time for Trading Week
      dtWeekEndExample        = D'2015.11.27 23:54:59';  // End Time for Trading Week
      
   _TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
      nameTimeFilterEnable    = "TimeFilterEnable (bool)",
      nameResetOnTimeFilter   = "ResetOnTimeFilter (bool)",
      nameWeekStartExample    = "WeekStartTime (datetime)",
      nameWeekEndExample      = "WeekEndTime (datetime)";
   datetime
      dtWeekOffset            = D'1970.01.04 00:00:00', // Week Start Correction Offset
      dtWeekStartMinor,                      // Minor Significant Week Start Date/Time
      dtWeekEndDelta,                        // Delta for Week End Date/Time
      dtWeekStartAdjusted,                   // Adjusted Week Start Date/Time
      dtWeekEndAdjusted;                     // Adjusted Week End Date/Time
   bool
      boolTimeValid           = true;        // Time for Trading is Valid


   //---- Define Timing Parameters and Variables
   input string
      strTimingSettings
         = "--- Timing Management Settings ---"; // Timing Management Settings
   extern bool
      boolSummerTrades        = true;  // Allow Trading in Summer (June-August)
   
   // Define Week Day Bitmap Slots
   #define  _DAYBIT_0d_    1
   #define  _DAYBIT_1d_    2
   #define  _DAYBIT_2d_    4
   #define  _DAYBIT_3d_    8
   #define  _DAYBIT_4d_    16
   #define  _DAYBIT_5d_    32
   #define  _DAYBIT_6d_    64

   // Define Day Hour Bitmap Slots
   #define  _HOURBIT_00h_  1
   #define  _HOURBIT_01h_  2
   #define  _HOURBIT_02h_  4
   #define  _HOURBIT_03h_  8
   #define  _HOURBIT_04h_  16
   #define  _HOURBIT_05h_  32
   #define  _HOURBIT_06h_  64
   #define  _HOURBIT_07h_  128
   #define  _HOURBIT_08h_  256
   #define  _HOURBIT_09h_  512
   #define  _HOURBIT_10h_  1024
   #define  _HOURBIT_11h_  2048
   #define  _HOURBIT_12h_  4096
   #define  _HOURBIT_13h_  8192
   #define  _HOURBIT_14h_  16384
   #define  _HOURBIT_15h_  32768
   #define  _HOURBIT_16h_  65536
   #define  _HOURBIT_17h_  131072
   #define  _HOURBIT_18h_  262144
   #define  _HOURBIT_19h_  524288
   #define  _HOURBIT_20h_  1048576
   #define  _HOURBIT_21h_  2097152
   #define  _HOURBIT_22h_  4194304
   #define  _HOURBIT_23h_  8388608

   extern int
      intWeekDayBitmap        =        // Day of Week for Trading
                                 _DAYBIT_0d_ |
                                 _DAYBIT_1d_ |
                                 _DAYBIT_2d_ |
                                 _DAYBIT_3d_ |
                                 _DAYBIT_4d_ |
                                 _DAYBIT_5d_ |
                                 _DAYBIT_6d_ |
                                 0,
      intDayHourBitmap        =        // Hour Bitmap for Trading
                                 _HOURBIT_00h_  |
                                 _HOURBIT_01h_  |
                                 _HOURBIT_02h_  |
                                 _HOURBIT_03h_  |
                                 _HOURBIT_04h_  |
                                 _HOURBIT_05h_  |
                                 _HOURBIT_06h_  |
                                 _HOURBIT_07h_  |
                                 _HOURBIT_08h_  |
                                 _HOURBIT_09h_  |
                                 _HOURBIT_10h_  |
                                 _HOURBIT_11h_  |
                                 _HOURBIT_12h_  |
                                 _HOURBIT_13h_  |
                                 _HOURBIT_14h_  |
                                 _HOURBIT_15h_  |
                                 _HOURBIT_16h_  |
                                 _HOURBIT_17h_  |
                                 _HOURBIT_18h_  |
                                 _HOURBIT_19h_  |
                                 _HOURBIT_20h_  |
                                 _HOURBIT_21h_  |
                                 _HOURBIT_22h_  |
                                 _HOURBIT_23h_  |
                                 0;
      
   input int
      intWeekDayTest          = -1,    // Day for Testing Optimisation of Week
      intDayHourTest          = -1;    // Hour for Testing Optimisation of Day

   _TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
      nameSummerTrades        = "SummerTrades (bool)",
      nameWeekDayBitmap       = "WeekDayBitmap (int)",
      nameDayHourBitmap       = "DayHourBitmap (int)";
   bool
      boolSignalTiming,          // Signal when Timing is Valid
      boolWeekDayBitmap[ 7 ],    // Timing Signal for Week Day
      boolDayHourBitmap[ 24 ];   // Timing Signal for Day Hour

      
   //---- Define Pip Adjustment Parameters and Variables
   //       ( Percentage in Point / Price Interest Point ) 
   input string
      strPipSizeSettings
         = "--- Pip Size Adjustment Settings ---"; // Pip Size Adjustment Settings
   extern double
      dblPipSizeAdjust        = -1;          // Pip Size (0 or less = AUTO)
   _TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
      namePipSizeAdjust       = "PipSizeAdjust (double)";
   double
      dblPipDelta,            // Current Size of a Pip
      dblPipMultiplier;       // Pip/Point Adjustment Multiplier
   int
      intPipDigits;           // Number of Decimal Digits for Pips
  
   
   //---- Define Spread Management Parameters and Variables
   input string
      strSpreadSettings
         = "--- Spread Management Settings ---";   // Spread Management Settings
   extern double
      dblSpreadMaximumPips    = 3.0;   // Maximum Spread Allowed (Pips)
   extern bool
      boolUseMaxSpread        = false; // Use Maximum Spread for Short Stop-Loss Offsets
   _TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
      nameSpreadMaximumPips   = "SpreadMaximumPips (double)",
      nameUseMaxSpread        = "UseMaxSpread (bool)";
   int
      intBidPoints,           // Current Bid Price (Points)
      intAskPoints,           // Current Ask Price (Points)
      intSpreadPoints,        // Current Spread Value (Points)
      intSpreadMaximumPoints; // Maximum Spread Allowed (Points)
   bool
      boolSignalSpread;       // Signal when Spread is below threshold

   
   //---- Define Slippage Management Parameters and Variables
   input string
      strSlippageSettings
         = "--- Slippage Management Settings ---"; // Slippage Management Settings
   extern double
      dblSlippageEnterPips    = 3.0,   // Slippage when Entering Trade (Pips)
      dblSlippageExitPips     = 3.0;   // Slippage when Exiting Trade (Pips)
   _TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
      nameSlippageEnterPips   = "SlippageEnterPips (double)",
      nameSlippageExitPips    = "SlippageExitPips (double)";
   int
      intSlippageEnterPoints, // Slippage when Entering Trade  (Points)
      intSlippageExitPoints;  // Slippage when Exiting Trade   (Points)


   //---- Define Lots Management Parameters and Variables
   input string
      strLotsSettings
         = "--- Lots Management Settings ---";  // Lots Management Settings
   extern double
      dblLotsFixed            = EMPTY;    // Use a Fixed Lot Size (0 or less = AUTO)
   _TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
      nameLotsFixed           = "LotsFixed (double)";

   
   //---- Define Money and Risk Management Parameters and Variables
   input string
      strMoneyRiskSettings
         = "--- Money & Risk Management Settings ---";   // Money & Risk Management Settings
   extern double
      dblRiskTiltRatio        = 0,        // Risk Tilt Ratio between Long & Short
      dblRiskMaximum          = 1.00,     // Maximum Risk per Order (Percentage)
      dblMarginMaximum        = 5.00;     // Maximum Margin per Order (Percentage)
   _TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
      nameRiskTiltRatio       = "RiskTiltRatio (double)",
      nameRiskMaximum         = "RiskMaximum (double)",
      nameMarginMaximum       = "MarginMaximum (double)";
   double
      dblRiskMaximumRatio,    // Maximum Risk per Order (Ratio)
      dblRiskMaxRatioShort,   // Maximum Risk per Long Order (Ratio)
      dblRiskMaxRatioLong,    // Maximum Risk per Short Order (Ratio)
      dblMarginMaximumRatio;  // Maximum Margin per Order (Ratio)   


   //---- Define Magic Numbers Parameters and Variables
   input string
      strMagicNumberSettings
         = "--- Magic Numbers Settings ---"; // Magic Numbers Settings
   extern int
      intMagicPrimary         = 0x0ABCDEF7;  // Primary Magic Number
   extern string
      strMagicIDTag           = "";          // Extra ID Tag for Global Variables
   _TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
      nameMagicPrimary        = "MagicPrimary (int)";


   //---- Define Global Variable Parameter Update Interval
   input string
      strGlobalVariablesSettings
         = "--- Global Variables Settings (Local) ---"; // Global Variables Settings (Local)
   input int
      intParamUpdateMins      = 5;     // Update Interval for Global Parameters
   input bool
      boolParamReset          = false; // Reset Global Parameters
   bool
      boolParamResetFlag      = false, // Flag to Control Reset of Global Parameters
      boolInvalidParameters   = false; // Check for Invallid Parameters
      
   //---- Define Parameters for Order Retry
   input string
      strOrderRetrySettings
         = "--- Order Retry Settings (Local) ---"; // Order Retry Settings (Local)
   input int
      intRetryCount           = 3;     // Order Update Retry Count
   input double
      dblSleepTimeMean        = 5.0,   // Retry Average Sleep Time
      dblSleepTimeMax         = 15.0;  // Retry Maximum Sleep Time


//+------------------------------------------------------------------------------------+
// Global Scope Enumerations and Variables
//+------------------------------------------------------------------------------------+

//---- Define Trading States Enumeration and Variables
enum
   ENUM_TRADING_STATE      // Define Trading States
      {
            _TRADING_FULL_ =  +1
         ,  _TRADING_EXIT_ =   0
         ,  _TRADING_NONE_ =  -1
      };
ENUM_TRADING_STATE
   enumTradingState        = _TRADING_FULL_  // Allow Trading on this EA
,  enumTradingStateAll     = _TRADING_FULL_  // Allow Trading on All EA's
;
double
   dblBalanceMinimumAll    = 0.00;           // Minimum Balance Requirement / Protection
_TGVNAMETAG_ // Define Terminal Global Variable Tags/Names
   nameTradingState        = "TradingState (int)",
   nameBalanceMinimum      = "BalanceMinimum (double)";

#define _ToPoints( dblPrice ) int( round( dblPrice / _Point ) )

//+------------------------------------------------------------------------------------+
// Prepare Global Context Variables Names with Chart Prefix
//+------------------------------------------------------------------------------------+
string
   namePrefix     = "",
   nameSymbol     = "Unknown Symbol",
   nameComment    = "";
double
   dblPointDelta,          // Current Size of Point
   dblTickDelta,           // Current Size of a Tick
   dblMarketLotMin,        // Market Minimum Lot Size
   dblMarketLotMax,        // Market Maximum Lot Size
   dblMarketLotStep,       // Market Lot Step
   dblMarketStopLevel,     // Market Minimum Stop Size (Delta)
   dblAccountDeposit;      // Initial Account Deposit

int
   intTickDeltaPoints,     // Current Size of a Tick (Point)
   intMarketStopLevel;     // Market Minimum Stop Size (Points)
string
   nameRenkoBasePriceObject,
   nameRenkoUpperLimitObject,
   nameRenkoLowerLimitObject,
   nameSignalResetButtonObject,
   nameSignalBuyButtonObject,
   nameSignalSellButtonObject;
string strTimeframe( ENUM_TIMEFRAMES enumTimeframePeriod )
{
   return( StringSubstr( EnumToString( enumTimeframePeriod ), 7 ) );
}
void voidGlobalContextPrepare()
{
   if( namePrefix != "")
   {
      if( nameSymbol == _Symbol ) return;
      voidDeleteObject( nameRenkoBasePriceObject );
      voidDeleteObject( nameRenkoUpperLimitObject );
      voidDeleteObject( nameRenkoLowerLimitObject );
      voidDeleteObject( nameSignalResetButtonObject );
      voidDeleteObject( nameSignalBuyButtonObject );
      voidDeleteObject( nameSignalSellButtonObject );
   }

   nameSymbol = _Symbol;
   
   string strVersion = _VERSION_;
   StringReplace( strVersion, ".", "" );
   
   if( StringLen( strMagicIDTag ) > 0 )
   {
      namePrefix  = _CODE_ + "." + strMagicIDTag + "." + _Symbol;
      nameComment = _CODE_ + strVersion + strMagicIDTag;
   }
   else
   {
      namePrefix  = _CODE_ + "." + _Symbol;
      nameComment = _CODE_ + strVersion;
   }
   
   //---- Get Market Information
   dblPointDelta        = _Point;

   dblTickDelta         = MarketInfo( _Symbol, MODE_TICKSIZE );
   intTickDeltaPoints   = _ToPoints(  dblTickDelta );

   dblMarketLotMin      = MarketInfo( _Symbol, MODE_MINLOT );
   dblMarketLotMax      = MarketInfo( _Symbol, MODE_MAXLOT );
   dblMarketLotStep     = MarketInfo( _Symbol, MODE_LOTSTEP );

   intMarketStopLevel   = (int) MarketInfo( _Symbol, MODE_STOPLEVEL );
   dblMarketStopLevel   = intMarketStopLevel * _Point;

   if( IsVisualMode() || !IsTesting() )
   {
      nameRenkoBasePriceObject      = StringConcatenate( namePrefix, ".", "Base" );
      nameRenkoUpperLimitObject     = StringConcatenate( namePrefix, ".", "Upper" );
      nameRenkoLowerLimitObject     = StringConcatenate( namePrefix, ".", "Lower" );
      nameSignalResetButtonObject   = StringConcatenate( namePrefix, ".", "Reset" );
      nameSignalBuyButtonObject     = StringConcatenate( namePrefix, ".", "Buy" );
      nameSignalSellButtonObject    = StringConcatenate( namePrefix, ".", "Sell" );
      
      if( ObjectCreate( 0, nameRenkoBasePriceObject, OBJ_HLINE, 0, 0, 0.0 ) )
      {
         ObjectSetInteger( 0, nameRenkoBasePriceObject, OBJPROP_COLOR, clrYellow );
         ObjectSetInteger( 0, nameRenkoBasePriceObject, OBJPROP_STYLE, STYLE_DOT );
      }
      else
         nameRenkoBasePriceObject = "";
         
      if( ObjectCreate( 0, nameRenkoUpperLimitObject, OBJ_HLINE, 0, 0, 0.0 ) )
      {
         ObjectSetInteger( 0, nameRenkoUpperLimitObject, OBJPROP_COLOR, clrAqua );
         ObjectSetInteger( 0, nameRenkoUpperLimitObject, OBJPROP_STYLE, STYLE_DOT );
      }
      else
         nameRenkoUpperLimitObject = "";
      
      if( ObjectCreate( 0, nameRenkoLowerLimitObject, OBJ_HLINE, 0, 0, 0.0 ) )
      {
         ObjectSetInteger( 0, nameRenkoLowerLimitObject, OBJPROP_COLOR, clrMagenta );
         ObjectSetInteger( 0, nameRenkoLowerLimitObject, OBJPROP_STYLE, STYLE_DOT );
      }
      else
         nameRenkoLowerLimitObject = "";
         
      voidCreateButton( 1, nameSignalResetButtonObject, "Reset", clrBlack, clrGold );
      voidCreateButton( 2, nameSignalBuyButtonObject,   "Buy",   clrWhite, clrDodgerBlue );
      voidCreateButton( 3, nameSignalSellButtonObject,  "Sell",  clrWhite, clrMediumVioletRed );
      voidButtonVisibility();

      voidCommentInitialise();
   }
}

// Delete Object
void voidDeleteObject( string &nameObject )
{
   if( nameObject != "" )
   {
      if( ObjectDelete( 0, nameObject ) )
         nameObject = "";
   }
}

// Create Button Object
void voidCreateButton( int intIndex, string &nameButton, string strText, color clrText, color clrButton )
{
   if( nameButton != "" )
   {
      if( ObjectCreate( 0, nameButton, OBJ_BUTTON, 0, 0, 0.0 ) )
      {
         ObjectSetString(  0, nameButton, OBJPROP_TEXT, strText ); 
      
         ObjectSetInteger( 0, nameButton, OBJPROP_CORNER, CORNER_RIGHT_UPPER );
         ObjectSetInteger( 0, nameButton, OBJPROP_XSIZE, 48 ); 
         ObjectSetInteger( 0, nameButton, OBJPROP_YSIZE, 20 );
         ObjectSetInteger( 0, nameButton, OBJPROP_XDISTANCE, 56 ); 
         ObjectSetInteger( 0, nameButton, OBJPROP_YDISTANCE, intIndex * 24 );
         
         ObjectSetInteger( 0, nameButton, OBJPROP_COLOR, clrText );
         ObjectSetInteger( 0, nameButton, OBJPROP_BGCOLOR, clrButton );
         ObjectSetInteger( 0, nameButton, OBJPROP_BORDER_COLOR, clrNONE );
      
         ObjectSetInteger( 0, nameButton, OBJPROP_BACK, false );
         ObjectSetInteger( 0, nameButton, OBJPROP_STATE, false );
         ObjectSetInteger( 0, nameButton, OBJPROP_SELECTABLE, false );
         ObjectSetInteger( 0, nameButton, OBJPROP_SELECTED, false );
         ObjectSetInteger( 0, nameButton, OBJPROP_HIDDEN, true );
      }
      else
         nameButton = "";
   }

   return;
}

void voidVisibleButton( string nameButton, bool boolVisible )
{
   if( nameButton != "" )
   {
      if( boolVisible )
      {
         ObjectSetInteger( 0, nameButton, OBJPROP_XSIZE, 48 ); 
         ObjectSetInteger( 0, nameButton, OBJPROP_YSIZE, 20 );
      }
      else
      {
         ObjectSetInteger( 0, nameButton, OBJPROP_XSIZE, 0 ); 
         ObjectSetInteger( 0, nameButton, OBJPROP_YSIZE, 0 );
      }
   }
}

void voidButtonVisibility()
{
   switch( enumTradePositions )
   {
      case _UP_LONG_:
         voidVisibleButton( nameSignalBuyButtonObject, true );
         voidVisibleButton( nameSignalSellButtonObject, false );
         break;
         
      case _DOWN_SHORT_:
         voidVisibleButton( nameSignalBuyButtonObject, false );
         voidVisibleButton( nameSignalSellButtonObject, true );
         break;
      
      default:
         voidVisibleButton( nameSignalBuyButtonObject, true );
         voidVisibleButton( nameSignalSellButtonObject, true );
   }            
}


//+------------------------------------------------------------------------------------+
// Renko: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalRenkoInitialise()
{
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      dblBrickSizePips           = dblGlobalVariableRead(
         nameBrickSizePips, dblBrickSizePips, namePrefix );
      dblReversalBrickRatio      = dblGlobalVariableRead(
         nameReversalBrickRatio, dblReversalBrickRatio, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values
   intRenkoBrickSizePoints
      = (int) round( fabs( dblBrickSizePips ) * dblPipMultiplier
      / intTickDeltaPoints ) * intTickDeltaPoints;
   
   if( intRenkoBrickSizePoints < intTickDeltaPoints )
      intRenkoBrickSizePoints = intTickDeltaPoints;
      
   dblBrickSizePips  = double( intRenkoBrickSizePoints ) / dblPipMultiplier;
   dblRenkoBrickSize = intRenkoBrickSizePoints * _Point;
   
   dblReversalBrickRatio  = fabs( dblReversalBrickRatio );
   intRenkoReversalPoints = (int) round(
         intRenkoBrickSizePoints * dblReversalBrickRatio );
   if( intRenkoReversalPoints < 1 )
      intRenkoReversalPoints = intRenkoBrickSizePoints;
}


//+------------------------------------------------------------------------------------+
// Stops Size: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalStopsInitialise()
{
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      enumATRTimeframe  = (ENUM_TIMEFRAMES) dblGlobalVariableRead(
         nameATRTimeframe, enumATRTimeframe, namePrefix );
      intATRPeriod      = (int) dblGlobalVariableRead(
         nameATRPeriod, intATRPeriod, namePrefix );
      dblATRFactor      = dblGlobalVariableRead(
         nameATRFactor, dblATRFactor, namePrefix );

      dblRiskStopSize   = dblGlobalVariableRead(
         nameRiskStopSize, dblRiskStopSize, namePrefix );
      dblStopLossSize   = dblGlobalVariableRead(
         nameStopLossSize, dblStopLossSize, namePrefix );
      dblTrailStopSize  = dblGlobalVariableRead(
         nameTrailStopSize, dblTrailStopSize, namePrefix );
      dblTrailStepSize  = dblGlobalVariableRead(
         nameTrailStepSize, dblTrailStepSize, namePrefix );
      dblBreakEvenSize  = dblGlobalVariableRead(
         nameBreakEvenSize, dblBreakEvenSize, namePrefix );
      dblTakeProfitSize = dblGlobalVariableRead(
         nameTakeProfitSize, dblTakeProfitSize, namePrefix );
      dblStealthGapSize = dblGlobalVariableRead(
         nameStealthGapSize, dblStealthGapSize, namePrefix );

      enumRiskStopType     = (ENUM_STOPSIZE_TYPE) dblGlobalVariableRead(
         nameRiskStopType, enumRiskStopType, namePrefix );
      enumStopLossType     = (ENUM_STOPSIZE_TYPE) dblGlobalVariableRead(
         nameStopLossType, enumStopLossType, namePrefix );
      enumTrailStopType    = (ENUM_STOPSIZE_TYPE) dblGlobalVariableRead(
         nameTrailStopType, enumTrailStopType, namePrefix );
      enumTrailStepType    = (ENUM_STOPSIZE_TYPE) dblGlobalVariableRead(
         nameTrailStepType, enumTrailStepType, namePrefix );
      enumBreakEvenType    = (ENUM_STOPSIZE_TYPE) dblGlobalVariableRead(
         nameBreakEvenType, enumBreakEvenType, namePrefix );
      enumTakeProfitType   = (ENUM_STOPSIZE_TYPE) dblGlobalVariableRead(
         nameTakeProfitType, enumTakeProfitType, namePrefix );
      enumStealthGapType   = (ENUM_STOPSIZE_TYPE) dblGlobalVariableRead(
         nameStealthGapType, enumStealthGapType, namePrefix );

      enumTrailingType     = (ENUM_TRAILING_TYPE) dblGlobalVariableRead(
         nameTrailingType, enumTrailingType, namePrefix );

      boolCloseOnReverseSignal   = (bool) dblGlobalVariableRead(
         nameCloseOnReverseSignal, boolCloseOnReverseSignal, namePrefix );
      boolCheckBarExtreme        = (bool) dblGlobalVariableRead(
         nameCheckBarExtreme, boolCheckBarExtreme, namePrefix );
      boolCompensateSpread       = (bool) dblGlobalVariableRead(
         nameCompensateSpread, boolCompensateSpread, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values
   if( intATRPeriod <  1 ) intATRPeriod = 1;
   if( dblATRFactor <= 0 )
   {
      if( ( enumATRTimeframe == PERIOD_CURRENT ) ||
          ( enumATRTimeframe == _Period        )    )
         dblATRFactor = 1.0;
      else
         dblATRFactor = sqrt( _Period / enumATRTimeframe );
   }
   
   voidCheckATRStops( true ); // Check ATR and also Update the Stop Sizes
}


//+------------------------------------------------------------------------------------+
// Check ATR and Update Stops Sizes if Necessary
//+------------------------------------------------------------------------------------+
void voidCheckATRStops( bool boolAlwaysUpdateStops = false )
{
   if( ( enumRiskStopType   == _RATIO_ATR_) ||
       ( enumStopLossType   == _RATIO_ATR_) ||
       ( enumTrailStopType  == _RATIO_ATR_) ||
       ( enumTrailStepType  == _RATIO_ATR_) ||
       ( enumBreakEvenType  == _RATIO_ATR_) ||
       ( enumTakeProfitType == _RATIO_ATR_) ||
       ( enumTakeProfitType == _RATIO_ATR_)    )
   {
      if( boolUpdateATRBase() )
         boolAlwaysUpdateStops = true;
   }

   if( boolAlwaysUpdateStops )
      voidUpdateStops(); // Update the Stops
}


//+------------------------------------------------------------------------------------+
// Update Stops Sizes
//+------------------------------------------------------------------------------------+
void voidUpdateStops()
{
   switch( enumTrailingType )
   {
      case _PROFIT_TRAILING_:
         boolTrailing            = true;
         boolTrailingAfterProfit = true;
         break;
   
      case _NORMAL_TRAILING_:
         boolTrailing            = true;
         boolTrailingAfterProfit = false;
         break;

      default:
         boolTrailing            = false;
         boolTrailingAfterProfit = false;
   }

   if( boolTrailing )
   {
      if( dblStopLossSize <= 0 )
      {
         dblStopLossSize   = dblReversalBrickRatio;
         enumStopLossType  = _RATIO_BRICK_;
      }

      if( dblTrailStopSize <= 0 )
      {
         dblTrailStopSize  = dblStopLossSize;
         enumTrailStopType = enumStopLossType;
      }
   }
   
   if( dblStopLossSize <= 0 )
   {
      boolCloseOnReverseSignal = true;
      dblStopLossSize          = EMPTY;
      intStopLossDeltaPoints   = EMPTY;
   }
   else
   {
      switch( enumStopLossType )
      {
         case _FIXED_PIPS_:
            intStopLossDeltaPoints = (int) round(
               dblStopLossSize * dblPipMultiplier );
            break;

         case _RATIO_ATR_:
            intStopLossDeltaPoints = (int) round(
               dblATRBase * dblStopLossSize / _Point );
            break;
         
         default: // case _RATIO_BRICK_:
            intStopLossDeltaPoints = (int) round(
               intRenkoBrickSizePoints * dblStopLossSize );
            break;
      }

      if( intStopLossDeltaPoints < intMarketStopLevel )
         intStopLossDeltaPoints = intMarketStopLevel;
   }

   if( dblRiskStopSize <= 0 )
   {
      dblRiskStopSize        = EMPTY;
      intRiskStopDeltaPoints = EMPTY;
   }
   else
   {
      switch( enumRiskStopType )
      {
         case _FIXED_PIPS_:
            intRiskStopDeltaPoints = (int) round(
               dblRiskStopSize * dblPipMultiplier );
            break;

         case _RATIO_ATR_:
            intRiskStopDeltaPoints = (int) round(
               dblATRBase * dblRiskStopSize / _Point );
            break;
         
         case _RATIO_BRICK_:
         default:
            intRiskStopDeltaPoints = (int) round(
               intRenkoBrickSizePoints * dblRiskStopSize );
            break;
      }

      if( intRiskStopDeltaPoints < intMarketStopLevel )
         intRiskStopDeltaPoints = intMarketStopLevel;
   }

   if( dblTrailStopSize <= 0 )
   {
      dblTrailStopSize        = EMPTY;
      intTrailStopDeltaPoints = EMPTY;
      boolTrailing            = false;
      boolTrailingAfterProfit = false;
   }
   else
   {
      switch( enumTrailStopType )
      {
         case _FIXED_PIPS_:
            intTrailStopDeltaPoints = (int) round(
               dblTrailStopSize * dblPipMultiplier );
            break;

         case _RATIO_ATR_:
            intTrailStopDeltaPoints = (int) round(
               dblATRBase * dblTrailStopSize / _Point );
            break;
         
         case _RATIO_BRICK_:
         default:
            intTrailStopDeltaPoints = (int) round(
               intRenkoBrickSizePoints * dblTrailStopSize );
            break;
      }

      if( intTrailStopDeltaPoints < intMarketStopLevel )
         intTrailStopDeltaPoints = intMarketStopLevel;
   }

   if( dblTrailStepSize <= 0 )
   {
      dblTrailStepSize        = EMPTY;
      intTrailStepDeltaPoints = 1;
   }
   else
   {
      switch( enumTrailStepType )
      {
         case _FIXED_PIPS_:
            intTrailStepDeltaPoints = (int) round(
               dblTrailStepSize * dblPipMultiplier );
            break;

         case _RATIO_ATR_:
            intTrailStepDeltaPoints = (int) round(
               dblATRBase * dblTrailStepSize / _Point );
            break;
         
         case _RATIO_BRICK_:
         default:
            intTrailStepDeltaPoints = (int) round(
               intRenkoBrickSizePoints * dblTrailStepSize );
            break;
      }

      if( intTrailStepDeltaPoints < 1 )
         intTrailStepDeltaPoints = 1;
   }

   if( dblBreakEvenSize <= 0 )
   {
      dblBreakEvenSize        = EMPTY;
      intBreakEvenDeltaPoints = EMPTY;
   }
   else
   {
      switch( enumBreakEvenType )
      {
         case _FIXED_PIPS_:
            intBreakEvenDeltaPoints = (int) round(
               dblBreakEvenSize * dblPipMultiplier );
            break;

         case _RATIO_ATR_:
            intBreakEvenDeltaPoints = (int) round(
               dblATRBase * dblBreakEvenSize / _Point );
            break;
         
         case _RATIO_BRICK_:
         default:
            intBreakEvenDeltaPoints = (int) round(
               intRenkoBrickSizePoints * dblBreakEvenSize );
            break;
      }

      if( intBreakEvenDeltaPoints < intMarketStopLevel )
         intBreakEvenDeltaPoints = intMarketStopLevel;
   }
   
   if( dblTakeProfitSize <= 0 )
   {
      dblTakeProfitSize        = EMPTY;
      intTakeProfitDeltaPoints = EMPTY;
   }
   else
   {
      switch( enumTakeProfitType )
      {
         case _FIXED_PIPS_:
            intTakeProfitDeltaPoints = (int) round(
               dblTakeProfitSize * dblPipMultiplier );
            break;

         case _RATIO_ATR_:
            intTakeProfitDeltaPoints = (int) round(
               dblATRBase * dblTakeProfitSize / _Point );
            break;
         
         case _RATIO_BRICK_:
         default:
            intTakeProfitDeltaPoints = (int) round(
               intRenkoBrickSizePoints * dblTakeProfitSize );
            break;
      }

      if( intTakeProfitDeltaPoints < intMarketStopLevel )
         intTakeProfitDeltaPoints = intMarketStopLevel;
   }
   
   if( dblStealthGapSize <= 0 )
   {
      dblStealthGapSize        = EMPTY;
      intStealthGapDeltaPoints = 0;
   }
   else
   {
      switch( enumStealthGapType )
      {
         case _FIXED_PIPS_:
            intStealthGapDeltaPoints = (int) round(
               dblStealthGapSize * dblPipMultiplier );
            break;

         case _RATIO_ATR_:
         case _RATIO_BRICK_:
         default:
            intStealthGapDeltaPoints = (int) round(
               intRenkoBrickSizePoints * dblStealthGapSize );
            break;
      }

      if( intStealthGapDeltaPoints < intMarketStopLevel )
         intStealthGapDeltaPoints = intMarketStopLevel;
   }

   if(   !boolTrailing && 
         (  ( intTakeProfitDeltaPoints == EMPTY ) ||
            ( intStopLossDeltaPoints   == EMPTY )    )  ) 
      boolCloseOnReverseSignal = true;
}


//+------------------------------------------------------------------------------------+
// Order Conditions: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalConditionsInitialise()
{
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      enumTradePositions         = (ENUM_TRADING_SIGNAL) dblGlobalVariableRead(
         nameTradePositions, enumTradePositions, namePrefix );
      intRangeDepthPeriod        = (int) dblGlobalVariableRead(
         nameRangeDepthPeriod, intRangeDepthPeriod, namePrefix );
      boolResetOnRangeFilter     = (bool) dblGlobalVariableRead(
         nameResetOnRangeFilter, boolResetOnRangeFilter, namePrefix );
      boolFadeSignal             = (bool) dblGlobalVariableRead(
         nameFadeSignal, boolFadeSignal, namePrefix );
      boolMultipleOrders         = (bool) dblGlobalVariableRead(
         nameMultipleOrders, boolMultipleOrders, namePrefix );
      boolRetriggerSignal        = (bool) dblGlobalVariableRead(
         nameRetriggerSignal, boolRetriggerSignal, namePrefix );
      boolOrderStacking          = (bool) dblGlobalVariableRead(
         nameOrderStacking, boolOrderStacking, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values
   if( intRangeDepthPeriod > 0 )
      boolRangeDepthFilter    = true;
   else
   {
      boolRangeDepthFilter    = false;
      boolResetOnRangeFilter  = false;
      intRangeDepthPeriod     = EMPTY;
      intRangeDepthHighPoints = EMPTY;
      intRangeDepthLowPoints  = EMPTY;
   }
}


//+------------------------------------------------------------------------------------+
// Time Filter: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
// 7 Days x 24 Hours x 60 Minutes x 60 Seconds
#define _7DAYS_ 604800
void voidGlobalTimeFilterInitialise()
{
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      boolTimeFilterEnable    = (bool) dblGlobalVariableRead(
         nameTimeFilterEnable, boolTimeFilterEnable, namePrefix );
      boolResetOnTimeFilter   = (bool) dblGlobalVariableRead(
         nameResetOnTimeFilter, boolResetOnTimeFilter, namePrefix );
      dtWeekStartExample      = (datetime) dblGlobalVariableRead(
         nameWeekStartExample, dtWeekStartExample, namePrefix );
      dtWeekEndExample        = (datetime) dblGlobalVariableRead(
         nameWeekEndExample, dtWeekEndExample, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values
   if( dtWeekStartExample >= dtWeekEndExample )
      boolTimeFilterEnable = false;
   else
   {
      if( boolTimeFilterEnable )
      {
         dtWeekStartMinor  = ( dtWeekStartExample - dtWeekOffset ) % _7DAYS_;
         dtWeekEndDelta    = dtWeekEndExample - dtWeekStartExample;
      }
   }
   
   if( !boolTimeFilterEnable ) boolResetOnTimeFilter = false;
}


//+------------------------------------------------------------------------------------+
// Timing Management: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalTimingInitialise()
{
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      boolSummerTrades  = (bool) dblGlobalVariableRead(
         nameSummerTrades, boolSummerTrades, namePrefix );
      intWeekDayBitmap  = (int) dblGlobalVariableRead(
         nameWeekDayBitmap, intWeekDayBitmap, namePrefix );
      intDayHourBitmap  = (int) dblGlobalVariableRead(
         nameDayHourBitmap, intDayHourBitmap, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values
   
   int i,j;
   
   if( ( intWeekDayTest >= 0 ) && ( intWeekDayTest < 7 ) )
   {
      ArrayInitialize( boolWeekDayBitmap, false );
      boolWeekDayBitmap[ intWeekDayTest ] = true;
   }
   else
   {
      for( i = 0, j = 1; i < 7; i++ )
      {
         boolWeekDayBitmap[ i ] = ( ( intWeekDayBitmap & j ) != 0 );
         j <<= 1;
      }
      
      if( ( intWeekDayTest >= 7 ) && ( intWeekDayTest < 14 ) )
         boolWeekDayBitmap[ intWeekDayTest - 7 ] = false;
   }
   
   if( ( intDayHourTest >= 0 ) && ( intDayHourTest < 24 ) )
   {
      ArrayInitialize( boolDayHourBitmap, false );
      boolDayHourBitmap[ intDayHourTest ] = true;
   }
   else
   {
      for( i = 0, j = 1; i < 24; i++ )
      {
         boolDayHourBitmap[ i ] = ( ( intDayHourBitmap & j ) != 0 );
         j <<= 1;
      }

      if( ( intDayHourTest >= 24 ) && ( intDayHourTest < 48 ) )
         boolDayHourBitmap[ intDayHourTest - 24 ] = false;
   }
}


//+------------------------------------------------------------------------------------+
// Pip Size Adjustment: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalPipSizeInitialise()
{
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      dblPipSizeAdjust  = dblGlobalVariableRead( namePipSizeAdjust,
                           dblPipSizeAdjust, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values
   if( dblPipSizeAdjust > 0 )
   {
      dblPipSizeAdjust  = pow( 10, round( log10( dblPipSizeAdjust ) ) );
      if( dblPipSizeAdjust < _Point )  dblPipSizeAdjust = _Point;

      dblPipDelta       = dblPipSizeAdjust;
      dblPipMultiplier  = dblPipDelta / _Point;
      intPipDigits      = (int) ( log10 ( dblPipDelta ) - log10( _Point ) );
   }
   else
   {
      dblPipSizeAdjust  = -1;
      dblPipMultiplier  = 1;
      dblPipDelta       = _Point;
      intPipDigits      = 0;
      switch( _Digits )
      {
         case 1:
         case 3:
         case 5:
            dblPipMultiplier = 10;
            dblPipDelta *= dblPipMultiplier;
            intPipDigits = 1;
      }
   }
}


//+------------------------------------------------------------------------------------+
// Spread Management: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalSpreadInitialise()
{
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      dblSpreadMaximumPips = dblGlobalVariableRead(
         nameSpreadMaximumPips, dblSpreadMaximumPips, namePrefix );
      boolUseMaxSpread     = (bool) dblGlobalVariableRead(
         nameUseMaxSpread, boolUseMaxSpread, namePrefix );                              
   }
   
   //---- Define Price Delta Variables for PIP Values
   intSpreadMaximumPoints
      = (int) round( fabs( dblSpreadMaximumPips ) * dblPipMultiplier );

   //---- Adjust and Correct Parameter Values                  
   dblSpreadMaximumPips = double( intSpreadMaximumPoints ) / dblPipMultiplier;
}


//+------------------------------------------------------------------------------------+
// Slippage Management: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalSlippageInitialise()
{
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      dblSlippageEnterPips = dblGlobalVariableRead( nameSlippageEnterPips,
                           dblSlippageEnterPips, namePrefix );
      dblSlippageExitPips     = dblGlobalVariableRead( nameSlippageExitPips,
                           dblSlippageExitPips, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values
   dblSlippageEnterPips    = fabs( dblSlippageEnterPips );
   dblSlippageExitPips     = fabs( dblSlippageExitPips );
   
   //---- Define Price Delta Variables for PIP Values
   intSlippageEnterPoints  = (int) round( dblSlippageEnterPips * dblPipMultiplier );
   intSlippageExitPoints   = (int) round( dblSlippageExitPips * dblPipMultiplier );
}


//+------------------------------------------------------------------------------------+
// Lots Management: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalLotsInitialise()
{
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      dblLotsFixed   =
         dblGlobalVariableRead( nameLotsFixed, dblLotsFixed, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values                  
   if( dblLotsFixed > 0 )
   {
      dblLotsFixed = round( dblLotsFixed / dblMarketLotStep ) * dblMarketLotStep;
      if( dblLotsFixed < dblMarketLotMin ) dblLotsFixed = dblMarketLotMin;
      if( dblLotsFixed > dblMarketLotMax ) dblLotsFixed = dblMarketLotMax;
   }
   else
      dblLotsFixed = EMPTY;
}


//+------------------------------------------------------------------------------------+
// Money & Risk Management: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalMoneyRiskInitialise()
{
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      dblRiskTiltRatio     = dblGlobalVariableRead(
         nameRiskTiltRatio, dblRiskTiltRatio, namePrefix );
      dblRiskMaximum       = dblGlobalVariableRead(
         nameRiskMaximum, dblRiskMaximum, namePrefix );
      dblMarginMaximum     = dblGlobalVariableRead(
         nameMarginMaximum, dblMarginMaximum, namePrefix );
   }
   
   //---- Adjust and Correct Variables
   dblRiskMaximumRatio     = fabs( dblRiskMaximum     ) / 100.0;
   dblMarginMaximumRatio   = fabs( dblMarginMaximum   ) / 100.0;

   double dblRiskDouble = dblRiskMaximumRatio * 2;
   if( dblRiskTiltRatio > 0 )
   {
      dblRiskMaxRatioShort = dblRiskDouble / ( 1 + dblRiskTiltRatio );
      dblRiskMaxRatioLong  = dblRiskDouble - dblRiskMaxRatioShort;
   }
   else
   {
      if( dblRiskTiltRatio < 0 )
      {
         dblRiskMaxRatioLong  = dblRiskDouble / ( 1 - dblRiskTiltRatio );
         dblRiskMaxRatioShort = dblRiskDouble - dblRiskMaxRatioLong;
      }
      else
      {
         dblRiskMaxRatioLong  = dblRiskMaximumRatio;
         dblRiskMaxRatioShort = dblRiskMaximumRatio;
      }
   }
}


//+------------------------------------------------------------------------------------+
// Magic Numbers: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalMagicInitialise()
{
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      intMagicPrimary   = (int) dblGlobalVariableRead( nameMagicPrimary,
                           intMagicPrimary, namePrefix );
   }
}


//+------------------------------------------------------------------------------------+
// Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalContextInitialise()
{
   voidGlobalPipSizeInitialise();      // Initialise Pip Size Settings
   voidGlobalRenkoInitialise();        // Initialise Renko Settings
   voidGlobalStopsInitialise();        // Initialise Stops Settings
   voidGlobalConditionsInitialise();   // Initialise Conditions Settings
   voidGlobalTimeFilterInitialise();   // Initialise Time Filter Management Settings
   voidGlobalTimingInitialise();       // Initialise Timing Management Settings
   voidGlobalSpreadInitialise();       // Initialise Spread Management Settings
   voidGlobalSlippageInitialise();     // Initialise Slippage Management Settings
   voidGlobalLotsInitialise();         // Initialise Lots Management Settings
   voidGlobalMoneyRiskInitialise();    // Initialise Money & Risk Management Settings
   voidGlobalMagicInitialise();        // Initialise Magic Numbers Settings
   
   //---- Only if in Live Trading
   if( !IsTesting() )
   {
      //---- Initialise & Retrieve Specific Variables
      enumTradingState
         = (ENUM_TRADING_STATE) dblGlobalVariableRead( nameTradingState, enumTradingState, namePrefix );

      //---- Initialise & Retrieve Overall Variables
      enumTradingStateAll
         = (ENUM_TRADING_STATE) dblGlobalVariableRead( nameTradingState, enumTradingStateAll, "" );
      dblBalanceMinimumAll
         = dblGlobalVariableRead( nameBalanceMinimum, dblBalanceMinimumAll, "" );
      
      GlobalVariablesFlush();
   }
   
   if( dblBalanceMinimumAll < 0 ) dblBalanceMinimumAll  = 0;

   boolParamResetFlag = false;
   
   if( IsVisualMode() || !IsTesting() )
      voidCommentInitialise();   // Initialise the Comment Display
}


//+------------------------------------------------------------------------------------+
// Display and Print Special Parameters
//+------------------------------------------------------------------------------------+
void voidCommentInitialise()
{
   if( IsVisualMode() || !IsTesting() )
   {
      if( dblPipDelta == 0 ) return;

      string strComment = StringConcatenate( nameComment,
         "; Spr: ", DoubleToString( intSpreadPoints / dblPipMultiplier, intPipDigits ),
         " (", DoubleToString( dblSpreadMaximumPips, intPipDigits ), ")",
         "; Brk: ", DoubleToString( dblBrickSizePips, intPipDigits ),
         "; SL: ", DoubleToString( dblStopLossSize, 3 ),
         "; BE: ", DoubleToString( dblBreakEvenSize, 3 ),
         "; TS: ", DoubleToString( dblTrailStopSize, 3 ),
         " (", DoubleToString( dblTrailStepSize, 3 ),
         "); TP: ", DoubleToString( dblTakeProfitSize, 3 ),
         "; Sig: ", enumSignalOpen );

      //voidCommentPrint( strComment );
      voidCommentPrintString( strComment, true, false );

      long
         longChartBars  = ChartGetInteger( 0, CHART_VISIBLE_BARS );
      double
         dblHigh  = ceil( High[ ArrayMaximum( High, (int) longChartBars, 0 ) ]
                  / dblRenkoBrickSize ) * dblRenkoBrickSize,
         dblLow   = floor( Low[  ArrayMinimum( Low,  (int) longChartBars, 0 ) ]
                  / dblRenkoBrickSize ) * dblRenkoBrickSize,
         dblTemp;

      if( intRenkoBasePricePoints > 0 )
      {
         if( nameRenkoBasePriceObject != "" )
            ObjectSetDouble( 0, nameRenkoBasePriceObject,
               OBJPROP_PRICE, intRenkoBasePricePoints * _Point );
      }
      else
         ObjectSetDouble( 0, nameRenkoBasePriceObject, OBJPROP_PRICE, 0.0 );
      
      if( intRenkoUpperLimitPoints > 0 )
      {
         dblTemp = intRenkoUpperLimitPoints * _Point;
         if( dblTemp > dblHigh ) dblHigh = dblTemp;
         if( nameRenkoUpperLimitObject != "" )
            ObjectSetDouble( 0, nameRenkoUpperLimitObject,
               OBJPROP_PRICE, dblTemp );
      }
      else
         ObjectSetDouble( 0, nameRenkoUpperLimitObject, OBJPROP_PRICE, 0.0 );

      if( intRenkoLowerLimitPoints > 0 )
      {
         dblTemp = intRenkoLowerLimitPoints * _Point;
         if( dblTemp < dblLow ) dblLow = dblTemp;
         if( nameRenkoLowerLimitObject != "" )
            ObjectSetDouble( 0, nameRenkoLowerLimitObject,
               OBJPROP_PRICE, dblTemp );
      }
      else
         ObjectSetDouble( 0, nameRenkoLowerLimitObject, OBJPROP_PRICE, 0.0 );
         
      ChartSetInteger( 0, CHART_AUTOSCROLL, true );
      ChartSetInteger( 0, CHART_SCALEFIX, true );
      ChartSetDouble(  0, CHART_FIXED_MAX, dblHigh + dblRenkoBrickSize );
      ChartSetDouble(  0, CHART_FIXED_MIN, dblLow  - dblRenkoBrickSize );
      ChartSetDouble(  0, CHART_PRICE_MAX, dblHigh );
      ChartSetDouble(  0, CHART_PRICE_MIN, dblLow );
      
      voidButtonVisibility();           
   }
}


//+------------------------------------------------------------------------------------+
// Startup Initialization function
//+------------------------------------------------------------------------------------+
bool
   boolStartInit = true // Procedural Initialisation Flags
;
void voidStartupInit()
{
   //---- Prepare Nameing and other Constants for Context Variables (Client Terminal)
   voidGlobalContextPrepare();      // Prepare prefix, names and other constants
   
   //---- Create and Define Global Context Variables (Client Terminal)
   voidGlobalContextInitialise();   // Initialize & retrieve values
   /*
   if( intRenkoTrailStopDeltaPoints > 0 )
   {
      if( intRenkoTakeProfitDeltaPoints > 0 )
      {
         if( intRenkoTrailStopDeltaPoints >= intRenkoTakeProfitDeltaPoints )
            boolInvalidParameters = true;
      }
   }
   */
   
   if( !boolInvalidParameters )
   {
      // Show or Hide Indicators depending on Environment
      HideTestIndicators( !( IsVisualMode() || !IsTesting() ) );
      
      // Initialise Renko Status
      voidInitRenko();
      voidCommentInitialise();
   }
}


//+------------------------------------------------------------------------------------+
// Tick Data Collection function
//+------------------------------------------------------------------------------------+
bool
   boolTickNew = false; // Flag when a new Tick is detected
bool boolTickDataCollect()
{
   // Collect Tick Data
   MqlTick mqlTickData;
   if( SymbolInfoTick( _Symbol, mqlTickData ) )
   {
      boolTickNew       = true;
      intBidPoints      = _ToPoints( mqlTickData.bid );
      intAskPoints      = _ToPoints( mqlTickData.ask );
      intSpreadPoints   = intAskPoints - intBidPoints;
   }
   else
      boolTickNew = false;
      
   return( boolTickNew );
}


//+------------------------------------------------------------------------------------+
// Align Renko Prices
//+------------------------------------------------------------------------------------+
int intRenkoAlign( int intPricePoints )
{
   return(
      (int) round( double( intPricePoints )
      / double( intRenkoBrickSizePoints ) ) * intRenkoBrickSizePoints );
}
double dblRenkoAlign( double dblPrice )
{
   return( round( dblPrice / dblRenkoBrickSize ) * dblRenkoBrickSize );
}


//+------------------------------------------------------------------------------------+
// Print Debug Info on Renk Status
//+------------------------------------------------------------------------------------+
void voidRenkoStatus( ENUM_TRADING_SIGNAL enumStatusSignal = _SIDE_OUT_ )
{
   if( IsTesting() )
   {
      string strStatus = "Renko Status - ";

      switch( enumStatusSignal )
      {
         case _UP_LONG_:
            strStatus = StringConcatenate( strStatus,
               "Down: ",  intRenkoDownCount, " (", intRenkoDownGainCount, ") " );
            break;
            
         case _DOWN_SHORT_:
            strStatus = StringConcatenate( strStatus,
               "Up: ",  intRenkoUpCount, " (", intRenkoUpGainCount, ") " );
            break;
      }
      
      int intTotal = intRenkoUpGainCount + intRenkoDownGainCount;
      double dblGain = AccountEquity() - dblAccountDeposit;
      strStatus = StringConcatenate( strStatus,
         "[", intTotal, " = ", DoubleToString( dblGain, 2 ), "]" );

      if( fabs( dblGain - intTotal ) > 1 )
         strStatus = StringConcatenate( strStatus, " - Warning!!!" ); 
         
      // voidPrint( strStatus );
   }
};


//+------------------------------------------------------------------------------------+
// Set the Renko Limits based on the new Base Price
//+------------------------------------------------------------------------------------+
bool boolRenkoStats = true;
void voidSetRenkoLimits( ENUM_TRADING_SIGNAL enumBarSignal = _SIDE_OUT_ )
{
   switch( enumBarSignal )
   {
      case _UP_LONG_:
         intRenkoBasePricePoints
            = intRenkoUpperLimitPoints - intRenkoBrickSizePoints;
         intRenkoLowerLimitPoints
            = intRenkoBasePricePoints - intRenkoReversalPoints;
         if( boolRenkoStats )
         {
            if( intRenkoUpCount == 0 ) voidRenkoStatus( enumBarSignal );
            
            intRenkoDownCount = 0;
            intRenkoUpCount++;
            if( intRenkoUpCount > 0 )
            {
               if( intRenkoUpCount > 1 )
                  intRenkoUpGainCount++;
               else
                  intRenkoUpGainCount -= 2;
            }
         }
         break;
   
      case _DOWN_SHORT_:
         intRenkoBasePricePoints
            = intRenkoLowerLimitPoints + intRenkoBrickSizePoints;
         intRenkoUpperLimitPoints
            = intRenkoBasePricePoints + intRenkoReversalPoints;
         if( boolRenkoStats )
         {
            if( intRenkoDownCount == 0 ) voidRenkoStatus( enumBarSignal );
            
            intRenkoUpCount = 0;
            intRenkoDownCount++;
            if( intRenkoDownCount > 0 )
            {
               if( intRenkoDownCount > 1 )
                  intRenkoDownGainCount++;
               else
                  intRenkoDownGainCount -= 2;
            }
         }
         break;

      default:
         intRenkoUpperLimitPoints   = intRenkoBasePricePoints
                                    + intRenkoBrickSizePoints;
         intRenkoLowerLimitPoints   = intRenkoBasePricePoints
                                    - intRenkoBrickSizePoints;
         intRenkoUpCount            = 0;
         intRenkoDownCount          = 0;
         intRenkoUpGainCount        = 0;
         intRenkoDownGainCount      = 0;
         break;               
   }
 }


//+------------------------------------------------------------------------------------+
// Update and Check Renko Bar and set Signals accordingly
//+------------------------------------------------------------------------------------+
bool boolCheckRenko( int intQuotePoints = EMPTY )
{
   bool boolSetSignal = false;
   enumSignalRenkoTrend = _SIDE_OUT_;

   if( intQuotePoints <= 0 )
      intQuotePoints = intBidPoints;
   else
      boolSetSignal = true;
   
   if( intRenkoBasePricePoints == 0 )
   {
      intRenkoBasePricePoints = intRenkoAlign( intQuotePoints );
      voidSetRenkoLimits();
   }
   else
   {
      if( intQuotePoints >= intRenkoUpperLimitPoints )
      {
         enumSignalRenkoTrend = _UP_LONG_;
         while( intQuotePoints >= intRenkoUpperLimitPoints )
            intRenkoUpperLimitPoints
               += intRenkoBrickSizePoints;
         voidSetRenkoLimits( enumSignalRenkoTrend );
      }
      
      if( intQuotePoints <= intRenkoLowerLimitPoints )
      {
         enumSignalRenkoTrend = _DOWN_SHORT_;
         while( intQuotePoints <= intRenkoLowerLimitPoints )
            intRenkoLowerLimitPoints
               -= intRenkoBrickSizePoints;
         voidSetRenkoLimits( enumSignalRenkoTrend );
      }
   }
   
   if( enumSignalRenkoTrend != _SIDE_OUT_ )
   {
      if( boolSetSignal )
      {
         if( !boolResetOnTimeFilter )
         {
            if( boolFadeSignal )
               enumSignalOpen = (ENUM_TRADING_SIGNAL) -int( enumSignalRenkoTrend );
            else
               enumSignalOpen = enumSignalRenkoTrend;
         }
      }
   }

   return( enumSignalRenkoTrend != _SIDE_OUT_ );
}


//+------------------------------------------------------------------------------------+
// Iniialise Renko by runinng through existing M1 Bars
//+------------------------------------------------------------------------------------+
void voidInitRenko()
{
   int intBars = _Period == PERIOD_M1 ? Bars : iBars( NULL, PERIOD_M1 );
   
   if( intBars > 0 )
   {
      boolRenkoStats = false;
      int i, intOpen, intHigh, intLow, intClose;
      for( i = Bars - 1; i >= 0; i-- )
      {
         if( _Period == PERIOD_M1 )
         {
            intOpen  = _ToPoints( Open[  i ] );
            intHigh  = _ToPoints( High[  i ] );
            intLow   = _ToPoints( Low[   i ] );
            intClose = _ToPoints( Close[ i ] );
         }
         else
         {
            intOpen  = _ToPoints( iOpen(  NULL, PERIOD_M1, i ) );
            intHigh  = _ToPoints( iHigh(  NULL, PERIOD_M1, i ) );
            intLow   = _ToPoints( iLow(   NULL, PERIOD_M1, i ) );
            intClose = _ToPoints( iClose( NULL, PERIOD_M1, i ) );
         }
         
         boolCheckRenko( intOpen );
         if( ( intHigh - intOpen ) <= ( intOpen - intLow ) )
         {
            boolCheckRenko( intHigh );
            boolCheckRenko( intLow );
         }
         else
         {
            boolCheckRenko( intLow );
            boolCheckRenko( intHigh );
         }
         boolCheckRenko( intClose );
      }
   }
   
   intRenkoUpCount            = 0;
   intRenkoDownCount          = 0;
   intRenkoUpGainCount        = 0;
   intRenkoDownGainCount      = 0;
   
   boolRenkoStats = true;
}


//+------------------------------------------------------------------------------------+
// Update ATR Details
//+------------------------------------------------------------------------------------+
bool boolUpdateATRBase()
{
   static datetime dtTimestamp = NULL;
   datetime dtCurrent = iTime( NULL, enumATRTimeframe, 0 );
   if( dtCurrent != dtTimestamp )
   {
      dtTimestamp = dtCurrent;
      dblATRBase  = iATR( NULL, enumATRTimeframe, intATRPeriod, 1 )
                  * dblATRFactor;
      return( true );
   }
   return( false );
};


//+------------------------------------------------------------------------------------+
// Update Range Depth
//+------------------------------------------------------------------------------------+
void voidUpdateRangeDepth()
{
   static datetime dtTimestamp = NULL;
   
   if( boolRangeDepthFilter )
   {
      datetime dtCurrent = Time[0];
      if( dtCurrent != dtTimestamp )
      {
         int intPeriod = ( Bars > intRangeDepthPeriod )
                       ? intRangeDepthPeriod : Bars - 1;

         intRangeDepthHighPoints = _ToPoints(
            High[ iHighest( NULL, 0, MODE_HIGH, intPeriod, 1 ) ] );
         intRangeDepthLowPoints = _ToPoints(
            Low[  iLowest(  NULL, 0, MODE_LOW,  intPeriod, 1 ) ] );
      }
   }
};


//+------------------------------------------------------------------------------------+
// Verify Range Depth Filter Conditions
//+------------------------------------------------------------------------------------+
bool boolCheckRangeDepth()
{
   if( boolRangeDepthFilter )
   {
      if( intBidPoints >= intRangeDepthLowPoints )
      {
         if( intBidPoints <= intRangeDepthHighPoints ) return( true );
      }
   }
   return( false );
}


//+------------------------------------------------------------------------------------+
// Verify Current Time Filter
//+------------------------------------------------------------------------------------+
bool boolCheckTimeFilter( bool boolUpdate = false )
{
   static int intPreviousStamp = EMPTY;
   if( !boolTimeFilterEnable )
   {
      boolTimeValid = true;
      return( true );
   }
   
   datetime dtCurrentDatetime = TimeCurrent();
   int intCurrentStamp = (int) dtCurrentDatetime / 3600;
   
   if( ( intCurrentStamp != intPreviousStamp ) || boolUpdate )
   {
      datetime dtCurrentMajor = ( dtCurrentDatetime - dtWeekOffset ) / _7DAYS_;
      dtWeekStartAdjusted     = ( dtCurrentMajor * _7DAYS_ ) + dtWeekStartMinor + dtWeekOffset;
      dtWeekEndAdjusted       = dtWeekStartAdjusted + dtWeekEndDelta;
      intPreviousStamp        = intCurrentStamp;
   }
   
   boolTimeValid  = (   ( dtCurrentDatetime >= dtWeekStartAdjusted ) &&
                        ( dtCurrentDatetime <= dtWeekEndAdjusted   )    );

   return( boolTimeValid );
}


//+------------------------------------------------------------------------------------+
// Verify Current Timing and set Signal accordingly
//+------------------------------------------------------------------------------------+
bool boolCheckTiming()
{
   if( !boolWeekDayBitmap[ DayOfWeek() ] || !boolDayHourBitmap[ Hour() ] )
   {
      boolSignalTiming = false;
      return( false );
   }
   
   if( !boolSummerTrades )
   {
      if( ( Month() >= 6 ) && ( Month() <= 8 ) )
      {
         boolSignalTiming = false;
         return( false );
      }
   }
      
   boolSignalTiming = true;
   return( true );
}


//+------------------------------------------------------------------------------------+
// Verify Current Spread and set Signal accordingly
//+------------------------------------------------------------------------------------+
bool boolCheckSpread( ENUM_TRADING_SIGNAL enumSignal )
{
   boolSignalSpread = ( intSpreadPoints <= intSpreadMaximumPoints );
   return( boolSignalSpread );
}


//+------------------------------------------------------------------------------------+
// Function to Process Indicator Signals and BUY/SELL Signals
//+------------------------------------------------------------------------------------+
ENUM_TRADING_SIGNAL
   enumSignalEnter = _SIDE_OUT_,
   enumSignalOpen  = _SIDE_OUT_;
bool boolProcessSignals()
{
   // Reset Signals
   enumSignalEnter = _SIDE_OUT_;
   
   if( IsVisualMode() || !IsTesting() ) voidCommentInitialise();
   
   if( boolCheckRenko() )           // Update and Check Renko Signals
   {
      if( enumSignalOpen == enumSignalRenkoTrend )
      {
         if( boolRetriggerSignal || boolOrderStacking )
         {
            int intTicket = EMPTY;
            switch( enumSignalOpen )
            {
               case _UP_LONG_:
                  intTicket = intTicketLong;
                  break;
               case _DOWN_SHORT_:
                  intTicket = intTicketShort;
                  break;
            }
            
            if( intTicket != EMPTY )
            {
               if( OrderSelect( intTicket, SELECT_BY_TICKET, MODE_TRADES ) )
               {
                  if( OrderCloseTime() == 0 )
                  {
                     if( boolOrderStacking )
                     {
                        double dblDelta = ( OrderOpenTime() - OrderStopLoss() )
                                        * enumSignalOpen;
                        if( dblDelta < 0 ) enumSignalOpen = _SIDE_OUT_;
                     }
                  }
                  else
                  {
                     if( boolRetriggerSignal )
                     {
                        if( OrderProfit() < 0 )
                        {
                           enumSignalOpen = _SIDE_OUT_;
                           intTicketLong  = EMPTY;
                           intTicketShort = EMPTY;
                        }
                     }
                  }
               }
            }
         }
      }

      if( enumSignalOpen != enumSignalRenkoTrend )
      {
         // Exit any open Reverse Orders
         if( boolCloseOnReverseSignal )
         {
            if( OrdersTotal() >  0 )
            {
               voidPrint( "Close Orders due to Signal Reversal!" );
               if( boolFadeSignal )
                  voidOrdersCloseDirection( enumSignalRenkoTrend, intMagicPrimary );
               else
                  voidOrdersCloseDirection( 
                     (ENUM_TRADING_SIGNAL) -int( enumSignalRenkoTrend ), intMagicPrimary );
            }
         }
         
         if( boolCheckTimeFilter() )   // Check Valid Trading Time
         {
            if( boolCheckSpread( enumSignalRenkoTrend ) )   // Check Spread
            {
               if( boolFadeSignal )
                  enumSignalEnter = (ENUM_TRADING_SIGNAL) -int( enumSignalRenkoTrend );
               else
                  enumSignalEnter = enumSignalRenkoTrend;
            }
            else
               voidPrint( StringConcatenate(
                  "Valid Renko Trend Signal (",  enumSignalRenkoTrend,
                  ") @", DoubleToString( Bid, _Digits ),
                  " with INVALID SPREAD (pips): ", DoubleToString(
                     double( intSpreadPoints ) / dblPipMultiplier, intPipDigits ),
                  "; Max: ", DoubleToString( dblSpreadMaximumPips, intPipDigits ) ) );
         }
         else
         {
            if( !boolResetOnTimeFilter )
            {
               if( boolFadeSignal )
                  enumSignalOpen = (ENUM_TRADING_SIGNAL) -int( enumSignalRenkoTrend );
               else
                  enumSignalOpen = enumSignalRenkoTrend;
            }
                  
            voidPrint( StringConcatenate(
               "Valid Renko Trend Signal (",  enumSignalRenkoTrend,
               ") @", DoubleToString( Bid, _Digits ),
               " cancelled due to Time Filtering: ",
               TimeToString( TimeCurrent() ),
               " (", TimeToString( dtWeekStartAdjusted ),
               " - ", TimeToString( dtWeekEndAdjusted ) ) );
         }
     }
   }
   
   return( enumSignalEnter != _SIDE_OUT_ );
}


//+------------------------------------------------------------------------------------+
// Process Bar function
//+------------------------------------------------------------------------------------+
void voidProcessBarTick()
{
   voidCheckATRStops();    // Check ATR and Update the Stop Sizes if necessary
   voidUpdateRangeDepth(); // Update Range Depth Filter Values
   
   if (  ( enumTradingStateAll != _TRADING_NONE_ )
      && ( enumTradingState    != _TRADING_NONE_ ) )
   {
      if( boolProcessSignals() )
      {
         if( enumSignalEnter != _SIDE_OUT_ )
            voidSignalOrder( enumSignalEnter );
      }
   }
   
   if( enumSignalOpen != _SIDE_OUT_ )
   {
      if( !boolCheckTimeFilter() )
      {
         if( OrdersTotal() >  0 )
         {
            voidPrint( "Closing All Orders due to Time Filter!" );
            voidOrdersCloseDirection( enumSignalOpen, intMagicPrimary );
         }
         if( boolResetOnTimeFilter )
            voidResetOpenSignal();
      }
   }
   
   voidOrdersProcessOnTick();    // Process Open Orders once per tick
}

//+------------------------------------------------------------------------------------+
// Reset Open Signal
//+------------------------------------------------------------------------------------+
void voidResetOpenSignal()
{
   voidPrint( "Reseting Signal due to Time Filter or Range Depth Filter!" );
   enumSignalOpen = _SIDE_OUT_;
   intTicketLong  = EMPTY;
   intTicketShort = EMPTY;
   voidCommentInitialise();
}


//+------------------------------------------------------------------------------------+
// Open Order based on Signal
//+------------------------------------------------------------------------------------+
void voidSignalOrder( ENUM_TRADING_SIGNAL enumOrderSignal, bool boolForced = false )
{
   enumSignalOpen = enumOrderSignal;

   if( !boolForced )
   {
      if( boolCheckRangeDepth() )
      {
         if( boolResetOnRangeFilter ) enumSignalOpen = _SIDE_OUT_;
         return;
      }
   }

   if(   ( enumTradePositions == _SIDE_OUT_      ) ||
         ( enumTradePositions == enumOrderSignal )    )
      intPlaceOrder( enumOrderSignal, boolForced );
   voidCommentInitialise();
}


//+------------------------------------------------------------------------------------+
// Check Reset Button State
//+------------------------------------------------------------------------------------+
void voidCheckResetButton()
{
   if( !IsVisualMode() ) voidPrint( "Reset Button Clicked" );
   if( bool( ObjectGetInteger( 0, nameSignalResetButtonObject, OBJPROP_STATE ) ) )
   {
      voidPrint( "Reset Button Depressed" );
      ObjectSetInteger( 0, nameSignalResetButtonObject, OBJPROP_STATE, false );
      voidResetOpenSignal();
   }
}


//+------------------------------------------------------------------------------------+
// Check Buy Button State
//+------------------------------------------------------------------------------------+
void voidCheckBuyButton()
{
   if( !IsVisualMode() ) voidPrint( "Buy Button Clicked" );
   if( bool( ObjectGetInteger( 0, nameSignalBuyButtonObject, OBJPROP_STATE ) ) )
   {
      voidPrint( "Buy Button Depressed" );
      ObjectSetInteger( 0, nameSignalBuyButtonObject, OBJPROP_STATE, false );
      voidSignalOrder( _UP_LONG_, true );
   }
}


//+------------------------------------------------------------------------------------+
// Check Sell Button State
//+------------------------------------------------------------------------------------+
void voidCheckSellButton()
{
   if( !IsVisualMode() ) voidPrint( "Sell Button Clicked" );
   if( bool( ObjectGetInteger( 0, nameSignalSellButtonObject, OBJPROP_STATE ) ) )
   {
      voidPrint( "Sell Button Depressed" );
      ObjectSetInteger( 0, nameSignalSellButtonObject, OBJPROP_STATE, false );
      voidSignalOrder( _DOWN_SHORT_, true );
   }
}


//+------------------------------------------------------------------------------------+
// Function to Calculate Stop Loss Size
//+------------------------------------------------------------------------------------+
int intCalcStopLossPoints( ENUM_TRADING_SIGNAL enumSignal )
{
   int intSLPoints
      = ( intStopLossDeltaPoints > 0 )
      ? intStopLossDeltaPoints
      : intRenkoReversalPoints;
      
   if( boolCheckBarExtreme )
   {
      int intSLExtreme = 0;
      switch( enumSignal )
      {
         case _UP_LONG_:
            intSLExtreme = intBidPoints
                         - _ToPoints( fmin( Low[ 0 ], Low[ 1 ] ) );
            break;
      
         case _DOWN_SHORT_:
            intSLExtreme = _ToPoints( fmax( High[ 0 ], High[ 1 ] ) )
                         - intBidPoints;
            break;
      }
      
      if( intSLExtreme > intSLPoints )
         intSLPoints = intSLExtreme;
   }
   
   return( intSLPoints );
}


//+------------------------------------------------------------------------------------+
// Function to Place Order
//+------------------------------------------------------------------------------------+
int
   intTicketLong  = EMPTY,
   intTicketShort = EMPTY;
int intPlaceOrder( ENUM_TRADING_SIGNAL enumSignal, bool boolForced = false )
{
   double dblRiskMaxRatio, dblLots, dblPrice;
   int intType, intColour, intPricePoints, intTicket;
      
   switch( enumSignal )
   {
      case _UP_LONG_:
         intTicket            = intTicketLong;
         dblRiskMaxRatio      = dblRiskMaxRatioLong;
         intPricePoints       = intAskPoints;
         intType              = OP_BUY;
         intColour            = MediumSeaGreen;
         break;
   
      case _DOWN_SHORT_:
         intTicket            = intTicketShort;
         dblRiskMaxRatio      = dblRiskMaxRatioShort;
         intPricePoints       = intBidPoints;
         intType              = OP_SELL;
         intColour            = MediumVioletRed;
         break;
      
      default:
         return( EMPTY );
   }
   
   if( !boolMultipleOrders && !boolForced )
   {
      if( intTicket != EMPTY )
      {
         if( OrderSelect( intTicket, SELECT_BY_TICKET, MODE_TRADES ) )
         {
            if( OrderCloseTime() == 0 ) return( intTicket );
         }
      }
   }
   
   if( boolCheckTiming() || boolForced )
   {
      int intStopDeltaPoints = ( ( intRiskStopDeltaPoints > 0 )
                             ? intRiskStopDeltaPoints
                             : intCalcStopLossPoints( enumSignal ) )
                             + intSpreadPoints;
   
      dblLots = dblLotsRisk( intStopDeltaPoints,
         dblRiskMaxRatio, dblMarginMaximumRatio );
   
      string
         strComment           = StringConcatenate( nameComment,
                                 boolForced ? "fS" : "S", intSpreadPoints );
      int
         intMagic             = intMagicPrimary,
         intOrderRetryCount   = IsTesting() ? 0 : intRetryCount;
      
      dblPrice = intPricePoints * _Point;
   
      do
      {
         intTicket = OrderSend(
               _Symbol, intType, dblLots, dblPrice,
               intSlippageEnterPoints, 0, 0, strComment,
               intMagic, 0, intColour );

         if( intTicket != EMPTY ) break;

         intOrderRetryCount--;
         if( intOrderRetryCount > 0 )
            voidSleepRandom( dblSleepTimeMean, dblSleepTimeMax );
      }
      while( intOrderRetryCount > 0 );

      if( intTicket == EMPTY )
      {
         voidPrint( StringConcatenate(
               "intPlaceOrder: OrderSend Error: ", ErrorDescription( GetLastError() )
               ) );
      }
   }
   else
      intTicket = EMPTY;

   switch( enumSignal )
   {
      case _UP_LONG_:
         intTicketLong = intTicket;
         break;
   
      case _DOWN_SHORT_:
         intTicketShort = intTicket;
         break;
   }
   
   return( intTicket );
}


//+------------------------------------------------------------------------------------+
// Function to Verify and Close Order for particular Ticket, Magic Number and Type
//+------------------------------------------------------------------------------------+
bool boolVerifyCloseOrder( int intTicket, int intMagic, int intType )
{
   if ( OrderSelect( intTicket, SELECT_BY_TICKET, MODE_TRADES ) )
   {
      if (     ( OrderSymbol()      == _Symbol  )
            && ( OrderMagicNumber() == intMagic    )
            && ( OrderType()        == intType     )  )
      {
         int intOrderRetryCount = IsTesting()?0:intRetryCount;
         
         if ( ( intType == OP_BUY ) || ( intType == OP_SELL ) )
         {
      	   do
            {
               if( OrderCloseTime() != 0 ) return( true );
               if( OrderClose( intTicket, OrderLots(), OrderClosePrice(),
                     intSlippageExitPoints, DeepPink ) )
                  return( true );
               intOrderRetryCount--;
               if( intOrderRetryCount > 0 )
                  voidSleepRandom( dblSleepTimeMean, dblSleepTimeMax );
            }
            while( intOrderRetryCount > 0 );
         }
         else
         {
            do
            {
               if( OrderDelete( intTicket ) ) return( true );
               intOrderRetryCount--;
               if( intOrderRetryCount > 0 )
                  voidSleepRandom( dblSleepTimeMean, dblSleepTimeMax );
            }
            while( intOrderRetryCount > 0 );
         }
            
         voidPrint( StringConcatenate(
            "boolVerifyCloseOrder: Order Close/Delete Error: ",
            ErrorDescription( GetLastError() )
            ) );
            
         return( false );
      }
      
      voidPrint( "boolVerifyCloseOrder: Invalid/Incorrect Symbol, Type or Magic number!" );
   }

   voidPrint( StringConcatenate(
      "boolVerifyCloseOrder: OrderSelect Error: ", ErrorDescription( GetLastError() )
      ) );
	            
   return( false );
}


//+------------------------------------------------------------------------------------+
//	Function to Close All Orders in a particular direction with specific Magic number
//+------------------------------------------------------------------------------------+
void voidOrdersCloseDirection( ENUM_TRADING_SIGNAL enumSignal, int intMagic )
{
	for( int i = OrdersTotal() - 1; i >= 0; i-- ) 
	{
   		if ( OrderSelect( i, SELECT_BY_POS, MODE_TRADES ) )
		{
			int intType = OrderType();
			
			switch( intType )
			{
				case OP_BUY:
				case OP_BUYLIMIT:
				case OP_BUYSTOP:
					if ( enumSignal == _DOWN_SHORT_ ) continue;
					break;
						
				case OP_SELL:
				case OP_SELLLIMIT:
				case OP_SELLSTOP:
					if ( enumSignal == _UP_LONG_ ) continue;
					break;
			}
			
			if(   ( OrderSymbol() 		== nameSymbol	)	&&
				   ( OrderMagicNumber()	== intMagic		)		)
			{
				int intTicket = OrderTicket();
				if( boolVerifyCloseOrder( intTicket, intMagic, intType ) )
				   voidPrint( StringConcatenate( "voidOrdersCloseDirectionMagic: ", intTicket ) );
			}
		}
	}
}


//+------------------------------------------------------------------------------------+
//	Function to Close All Orders for particular Magic Number
//+------------------------------------------------------------------------------------+
void voidOrdersCloseAll( int intMagic )
{
	voidPrint( "Closing All Orders" );
	
	for( int i = OrdersTotal() - 1; i >= 0; i-- ) 
	{
   		if( OrderSelect( i, SELECT_BY_POS, MODE_TRADES ) )
		{
			if( OrderSymbol() == _Symbol )
			{
				if( OrderMagicNumber() == intMagic )
					boolVerifyCloseOrder( OrderTicket(), intMagic, OrderType() );
			}
		}
	}
}


//+------------------------------------------------------------------------------------+
// Function to Process Open Orders on Every Tick one at a time
//+------------------------------------------------------------------------------------+
void voidOrdersProcessOnTick()
{
   // Static Variables
   static int intIndex = 0;
   
   intIndex--; // Next Order
   
   if( intIndex < 0 )
   {
      intIndex = OrdersTotal() - 1; // Restart from End
      if( intIndex < 0 ) return;
   }

   if( OrderSelect( intIndex, SELECT_BY_POS, MODE_TRADES ) )
   {
      // Check if Open
      if( OrderCloseTime() != 0 ) return;

      // Check if Right Symbol
      if( OrderSymbol() != _Symbol )   return;
         
      // Check Magic Number
      int intMagic = OrderMagicNumber();
      if( intMagic != intMagicPrimary ) return;
            
      // Check Order Type
      ENUM_TRADING_SIGNAL enumDirection;
      int intType  = OrderType();
      switch( intType )
      {
         case OP_BUY:
            enumDirection = _UP_LONG_;
            break;
      
         case OP_SELL:
            enumDirection = _DOWN_SHORT_;
            break;
            
         default:
            return;
      }
      
      bool
         boolUpdate           = false,
         boolClose            = false;
      int
         intTPPoints          = 0, 
         intSLPoints          = 0,
         intTakeProfitPoints  = 0,
         intStopLossPoints    = 0;
      double
         dblOpen              = OrderOpenPrice();

      // Check Time Filter to see if Order needs to be Closed
      if( !boolCheckTimeFilter() )
      {
         voidPrint( "Closing Order due to Time Filtering!" );
         boolClose = true;
      }
      else
      {
         int
            intOpenPoints        = _ToPoints( dblOpen ),
            intSpreadOffsetPoints
               = boolUseMaxSpread ? intSpreadMaximumPoints : intSpreadPoints,
            intMinStopPoints     = intMarketStopLevel + intSpreadOffsetPoints;
         
         // Check Take-Profit
         intTPPoints         = _ToPoints( OrderTakeProfit() );
         intTakeProfitPoints = intTPPoints
                             - ( ( intTPPoints > 0 )
                             ? ( intStealthGapDeltaPoints * enumDirection )
                             : 0 );
         if( intTakeProfitDeltaPoints > 0 )
         {
            if( intTakeProfitPoints <= 0 )
            {
               intTakeProfitPoints = intOpenPoints
                  + intTakeProfitDeltaPoints * enumDirection;
               boolUpdate = true;
            }
         }
         
         // Check Stop-Loss
         intSLPoints       = _ToPoints( OrderStopLoss() );
         intStopLossPoints = intSLPoints
                           + ( ( intSLPoints > 0 )
                           ? ( intStealthGapDeltaPoints * enumDirection )
                           : 0 );
                           
         if( intStopLossPoints == 0 )
         {
            intStopLossPoints = intOpenPoints
               - ( intCalcStopLossPoints( enumDirection )
               + intSpreadOffsetPoints ) * enumDirection;
            if( boolCompensateSpread )
               intStopLossPoints += ( intSpreadOffsetPoints / 2 ) * enumDirection;
            boolUpdate = true;
         }
         
         // Check Break-Even & Trailing Stop
         int intClosePoints = _ToPoints( OrderClosePrice() );
         if( boolTrailing || ( intBreakEvenDeltaPoints > 0 ) )
         {
            int intTrailingPoints = ( intClosePoints - intOpenPoints ) * enumDirection;
            if( intTrailingPoints > 0 )
            {
               if( intBreakEvenDeltaPoints > 0 )
               {
                  if( intTrailingPoints >= intBreakEvenDeltaPoints )
                  {
                     int intStopPoints = ( intStopLossPoints - intOpenPoints ) * enumDirection;
                     if( intStopPoints < 0 )
                     {
                        intStopLossPoints = boolCompensateSpread
                                          ? intOpenPoints + intSpreadOffsetPoints * enumDirection
                                          : intOpenPoints;
                        /*
                        if( boolOrderStacking )
                        {
                           if( intRenkoStopLossDeltaPoints > 0 )
                              intStopLossPoints += intRenkoStopLossDeltaPoints * enumDirection;
                        }
                        */
                        boolUpdate = true;
                     }
                  }
               }

               if( boolTrailing )
               {
                  // Calculate Trailing Stop
                  int intTrailStepPoints = intTrailStepDeltaPoints;
                  if( intTrailStepPoints < 1 ) intTrailStepPoints = 1;
                  intTrailingPoints
                     = ( int( floor( intTrailingPoints / intTrailStepPoints ) )
                     * intTrailStepPoints - intTrailStopDeltaPoints )
                     * enumDirection + intOpenPoints;
                  if( boolCompensateSpread )
                     intTrailingPoints
                        -= ( intSpreadOffsetPoints / 2 ) * enumDirection;
   
                  // Adjust Trailing Stops
                  switch( enumDirection )
                  {
                     case _UP_LONG_:
                        if( intTrailingPoints > intStopLossPoints )
                        {
                           if(   !boolTrailingAfterProfit ||
                                 ( intTrailingPoints >= intOpenPoints ) )
                           {
                              if( intClosePoints > ( intTrailingPoints + intMinStopPoints ) )
                              {
                                 intStopLossPoints = intTrailingPoints;
                                 boolUpdate        = true;
                              }
                           }
                        }
                        break;
                  
                     case _DOWN_SHORT_:
                        if( intTrailingPoints < intStopLossPoints )
                        {
                           if(   !boolTrailingAfterProfit ||
                                 ( intTrailingPoints < intOpenPoints ) )
                           {
                              if( intClosePoints < ( intTrailingPoints - intMinStopPoints ) )
                              {
                                 intStopLossPoints = intTrailingPoints;
                                 boolUpdate        = true;
                              }
                           }
                        }
                        break;
                  }
               }
            }
         }

         // Check if Order has hit Stop-Loss and needs to be Closed
         if( intStopLossPoints > 0 )
         {
            bool boolHitSL = false;
            switch( enumDirection )
            {
               case _UP_LONG_:
                  boolHitSL = ( intClosePoints <= intStopLossPoints );
                  break;
            
               case _DOWN_SHORT_:
                  boolHitSL = ( intClosePoints >= intStopLossPoints );
                  break;
            }
            
            if( boolHitSL )
            {
               voidPrint( StringConcatenate(
                  "voidOrdersProcessOnTick: Reached Stop-Loss @", intStopLossPoints ) );
               boolClose = true;
            }
         }
      
         // Check if Order has hit Take-Profit and needs to be Closed
         if( intTakeProfitPoints > 0 )
         {
            bool boolHitTP = false;
            switch( enumDirection )
            {
               case _UP_LONG_:
                  boolHitTP = ( intClosePoints >= intTakeProfitPoints );
                  break;
            
               case _DOWN_SHORT_:
                  boolHitTP = ( intClosePoints <= intTakeProfitPoints );
                  break;
            }
            
            if( boolHitTP )
            {
               voidPrint( StringConcatenate(
                  "voidOrdersProcessOnTick: Reached Take-Profit @", intTakeProfitPoints ) );
               boolClose = true;
            }
         }
      }

      // Check if Order needs to be Closed
      if( boolClose )
      {
         voidPrint( "voidOrdersProcessOnTick: Close Order" );
         boolUpdate = !boolVerifyCloseOrder( OrderTicket(), intMagic, intType );
      }
      
      if( boolUpdate )
      {
         if( intStopLossPoints   > 0 ) 
            intStopLossPoints   -= intStealthGapDeltaPoints * enumDirection;
         if( intTakeProfitPoints > 0 )
            intTakeProfitPoints += intStealthGapDeltaPoints * enumDirection;
         
         if(   ( intSLPoints != intStopLossPoints   ) ||
               ( intTPPoints != intTakeProfitPoints )    )
         {
            double
               dblStopLoss   = dbPriceNormalize( intStopLossPoints   * _Point ),
               dblTakeProfit = dbPriceNormalize( intTakeProfitPoints * _Point );

            int intOrderRetryCount = IsTesting() ? 0 : intRetryCount;
            
            do
            {
               if( OrderModify( OrderTicket(), dblOpen,
                     dblStopLoss, dblTakeProfit, OrderExpiration(), Gold ) )
                  return;
               intOrderRetryCount--;
               if( intOrderRetryCount > 0 )
                  voidSleepRandom( dblSleepTimeMean, dblSleepTimeMax );
            }
            while( intOrderRetryCount > 0 );
            
            voidPrint( StringConcatenate(
               "voidOrdersProcessOnTick: OrderModify Error",
               "; Direction=", enumDirection,
               "; SL=", DoubleToString( dblStopLoss, Digits ),
               "; TP=", DoubleToString( dblTakeProfit, Digits ),
               "; Ask=", DoubleToString( Ask, Digits ),
               "; Bid=", DoubleToString( Bid, Digits ),
               "; Error: ", ErrorDescription( GetLastError() )
               ) );
         }
      }
   }
}


//+------------------------------------------------------------------------------------+
// Process Statistics for Deinitialisation
//+------------------------------------------------------------------------------------+
double dblOnTesterValue;
void voidProcessStats()
{
   int intTradesTotal = (int) TesterStatistics( STAT_TRADES );
   
   if( intTradesTotal > 0 )
      dblOnTesterValue  = 100.0 * (double) TesterStatistics( STAT_PROFIT_TRADES )
                        / (double) intTradesTotal;
   else
      dblOnTesterValue  = 0;
      
   voidPrint( StringConcatenate(
      "Renko Gain Count: ", intRenkoUpGainCount + intRenkoDownGainCount,
      "; Up:", intRenkoUpGainCount, "; Down: ", intRenkoDownGainCount ) );

   return;
}


//+------------------------------------------------------------------------------------+
// Tester Event Function
//+------------------------------------------------------------------------------------+

double OnTester()
{
   voidProcessStats();  // Process and Log Final Statistics
   return( dblOnTesterValue );
}


//+------------------------------------------------------------------------------------+
// Check minimum Trading Conditions
//+------------------------------------------------------------------------------------+
bool boolTradingConditionsValid()
{
   //---- Check if Connected
   if ( !IsConnected() )
   {
      voidCommentPrint("NOT Trading (NOT connected)!");
      return(false);
   }

   //---- Check if Terminal Allows Trading
   /*if  ( !IsTradeAllowed() )
   {
      voidCommentPrint("NOT Trading (Disallowed by terminal)!");
      return(false);
   }*/
   
   //---- Check Global Overall Trading State 
   if ( enumTradingStateAll == _TRADING_NONE_ )
   {
      voidCommentPrint("NOT Trading (Overall trading state disabled)!");
      return(false);
   }
   
   //---- Check Global Current Trading State 
   if ( enumTradingState == _TRADING_NONE_ )
   {
      voidCommentPrint("NOT Trading (Current trading state disabled)!");
      return(false);
   }

   return(true);  // Otherwise, proceed with trading
}


//+-----------------------------------------------------------------------+
// Check minimum Trade Entering Conditions
//+-----------------------------------------------------------------------+
bool boolTradeEnterConditionsValid()
{
   //---- If Testing, then ignore minimum requirements
   if( IsTesting() )
      return(true);

   //---- Check if Connected
   if( !IsConnected() )
   {
      voidCommentPrint("NOT Entering Trade (NOT connected)!");
      return(false);
   }

   //---- Check if Terminal Allows Trading
   /*if  ( !IsTradeAllowed() )
   {
      voidCommentPrint("NOT Entering Trade (Disallowed by terminal)!");
      return(false);
   }*/
   
   //---- Check Global Overall Trading State 
   if ( enumTradingStateAll != _TRADING_FULL_ )
   {
      voidCommentPrint("NOT Entering Trade (Overall trading state)!");
      return(false);
   }
   
   //---- Check Global Current Trading State 
   if ( enumTradingState != _TRADING_FULL_ )
   {
      voidCommentPrint("NOT Entering Trade (Current trading state)!");
      return(false);
   }

   //---- Check if Accound Balance is acceptable
   double dblValueAccount
      = fmin( fmin( AccountEquity(), AccountBalance() ), AccountFreeMargin() );
   if ( dblValueAccount < dblBalanceMinimumAll )
   {
      voidCommentPrint("NOT Entering Trade (Balance too low)!");
      return(false);
   }
   
   return(true);  // Otherwise, proceed with entering trade
}


//+------------------------------------------------------------------------------------+
// Initialisation Event Function
//+------------------------------------------------------------------------------------+
int OnInit()
{
   // Allow for remainder of initialisation to be done on the first tick
   boolStartInit = true;
   
   // Check if Global Parameters need to be Reset
   boolParamResetFlag = boolParamReset;
   
   // Set Interval for Updateing Global Parameters
   if( !IsTesting() )
   {
      if( !EventSetTimer( intParamUpdateMins * 60 ) )
         voidPrint( "Error: Unable to Set Time for Global Parameter Update Interval!" );
   }
   
   // Do Startup Initialisation
   dblAccountDeposit = AccountBalance();
   voidStartupInit();
   
   if( boolInvalidParameters )
      return( INIT_PARAMETERS_INCORRECT );
   
   return( INIT_SUCCEEDED );  // Initialisation complete
}


//+------------------------------------------------------------------------------------+
// Tick Event Function
//+------------------------------------------------------------------------------------+
void OnTick()
{
   // Collect Tick Data
   boolTickDataCollect();
   
   // Do Startup Initialisation
   if ( boolStartInit )
   {
      voidStartupInit();
      boolStartInit = false;
   }
   
   // Only needed in Real Execution, not in Testing Mode
   if ( !IsTesting() )
   {
      //---- Check Minimum Requiments or Conditions for Trading
      if ( !boolTradingConditionsValid() )
         return;  // Do NOT trade, Conditions are not Valid
   }
   else
   {
      if( IsVisualMode() )
      {
         // Check Chart Buttons in visual Mode
         voidCheckResetButton();
         voidCheckBuyButton();
         voidCheckSellButton();
      }
   }
   
   //---- Process Details for New Bar and Every Tick positions
   voidProcessBarTick();  // Process for new bar and tick
   
   //--- Processing Complete
   return;
}


//+------------------------------------------------------------------------------------+
// Timer Event Function
//+------------------------------------------------------------------------------------+
void OnTimer()
{
   if( boolStartInit )
   {
      voidStartupInit();
      boolStartInit = false;
   }
   else
      voidGlobalContextInitialise();   // Initialize & retrieve values
}

//+------------------------------------------------------------------------------------+
// Chart Event Function
//+------------------------------------------------------------------------------------+
void OnChartEvent(const int id,         // Event ID 
                  const long& lparam,   // Parameter of type long event 
                  const double& dparam, // Parameter of type double event 
                  const string& sparam  // Parameter of type string events
                  ) 
{ 
   //voidPrint( StringConcatenate( "Chart Event: ", sparam ) );
   
   //--- Check if Chart Object is Clicked by mouse
   if( id == CHARTEVENT_OBJECT_CLICK ) 
   { 
      // Check if specific buttons were clicked

      if( sparam == nameSignalResetButtonObject )  voidCheckResetButton();
      if( sparam == nameSignalBuyButtonObject )    voidCheckBuyButton();
      if( sparam == nameSignalSellButtonObject )   voidCheckSellButton();
   }         
} 


//+------------------------------------------------------------------------------------+
// Deinitialisation Function
//+------------------------------------------------------------------------------------+
void OnDeinit( const int intReason )
{
   voidDeleteObject( nameRenkoBasePriceObject );
   voidDeleteObject( nameRenkoUpperLimitObject );
   voidDeleteObject( nameRenkoLowerLimitObject );
   voidDeleteObject( nameSignalResetButtonObject );
   voidDeleteObject( nameSignalBuyButtonObject );
   voidDeleteObject( nameSignalSellButtonObject );

   ChartSetInteger( 0, CHART_SCALEFIX, false );

   EventKillTimer(); // Killer the Timer Event
   return;  // Deinitialisation complete
}


//+------------------------------------------------------------------------------------+
//	Functions for Normalising Price
//+------------------------------------------------------------------------------------+

	// Factor and Normalise the Lot 
	double dbPriceNormalize( double dblPrice )
	{
		return( NormalizeDouble(
		   round( dblPrice / dblTickDelta ) * dblTickDelta, Digits ) );
	}


//+------------------------------------------------------------------------------------+
//	Functions for Normalising Volume or Lot Sizing
//+------------------------------------------------------------------------------------+

	// Calculate Max Lot Size based on Maximum Risk
	double dblLotsRisk(
	   int intStopDeltaPoints, double dblRiskRatio, double dblMarginRatio )
	{
		if( dblLotsFixed > 0 ) return( dblLotsFixed );
		
		double
			dblValueAccount
			   = fmin( fmin( AccountEquity(), AccountBalance() ), AccountFreeMargin() ),
			dblValueRisk   = dblValueAccount * dblRiskRatio,
			dblValueMargin = dblValueAccount * dblMarginRatio,
			dblMarginOrder = MarketInfo( _Symbol, MODE_MARGINREQUIRED ),
			dblLossOrder   =
			   round( intStopDeltaPoints * _Point / dblTickDelta )
			   * MarketInfo( _Symbol, MODE_TICKVALUE ),
			dblCalcLotLoss
				= floor( dblValueRisk   / dblLossOrder    / dblMarketLotStep )
				* dblMarketLotStep,
			dblCalcLotMargin
				= floor( dblValueMargin / dblMarginOrder  / dblMarketLotStep )
				* dblMarketLotStep,
			dblCalcLot = fmin( dblCalcLotLoss, dblCalcLotMargin );
		
		if ( dblCalcLot < dblMarketLotMin ) dblCalcLot = dblMarketLotMin;
		if ( dblCalcLot > dblMarketLotMax ) dblCalcLot = dblMarketLotMax;

		return ( dblCalcLot );
	}
	
	// Calculate Max Stop Loss Size based on Maximum Risk
	double dblStopLossRisk(
	   double dblLots, double dblRiskRatio, double dblMarginRatio )
	{
		double
			dblValueAccount
			   = fmax( fmax( AccountEquity(), AccountBalance() ), AccountFreeMargin() ),
			dblValueRisk      = dblValueAccount * dblRiskRatio,
			dblStopLossDelta  = dblValueRisk * dblTickDelta / ( dblLots
			                  * MarketInfo( _Symbol, MODE_TICKVALUE ) ),
			dblCalcStopLoss   = dbPriceNormalize( dblStopLossDelta );
			
		if( dblCalcStopLoss < dblMarketStopLevel )
		   dblCalcStopLoss = dblMarketStopLevel ;

		return ( dblCalcStopLoss );
	}
	
	
//+------------------------------------------------------------------------------------+
//	Variables and Functions for Handling available Time Frame Periods
//+------------------------------------------------------------------------------------+

	// Define Timeframe Values as per defined Contants
	int
		intTimeframeValues[] = {
			PERIOD_M1,  PERIOD_M5,  PERIOD_M15, PERIOD_M30,
			PERIOD_H1,  PERIOD_H4,  PERIOD_D1,  PERIOD_W1, PERIOD_MN1
		};
                     
	// Define Timeframe Period Names/Codes for above Values
	string
		strTimeframeNames[] = {
			"M1", "M5", "M15", "M30", "H1", "H4", "D1", "W1", "MN1"
		};
	
	// Function to Return Period Index based on Period Value
	int intTimeframeIndex( int intTimeframeValue )
	{
		int intPos = ArrayBsearch( intTimeframeValues, intTimeframeValue );
		if ( intTimeframeValues[intPos] != intTimeframeValue )
			return( EMPTY );
		return( intPos );
	}
	
	// Function to Return Timeframe Period Value base on Timeframe Period Index
	int intTimeframeValue( int intTimeframeIndex )
	{
		if(   ( intTimeframeIndex < 0 ) ||
		      ( intTimeframeIndex >= ArraySize( intTimeframeValues ) ) )
		      return NULL;
		      
		return( intTimeframeValues[ intTimeframeIndex ] );  
	}

	// Function to Return Timeframe Period Name based on Timeframe Period Value
	string strTimeframeName( int intTimeframeValue )
	{
		int intPos = ArrayBsearch( intTimeframeValues, intTimeframeValue );
		if ( intTimeframeValues[intPos] != intTimeframeValue )
			return("Unknown?");
		return( strTimeframeNames[intPos] );
	}
	
   // Function to Correct Timeframe Period or Convert from Index	
   int intTimeframeCorrect( int intTimeframeRef )
   {
      if( intTimeframeRef < 0 )
         return( intTimeframeValue( -intTimeframeRef - 1 ) );

      return( intTimeframeValue( intTimeframeIndex( intTimeframeRef ) ) );
   }
	


//+------------------------------------------------------------------------------------+
//	Variables and Functions for Handling Global Context Variables
//+------------------------------------------------------------------------------------+

	// Variable to Identify when Global Context has changed ( Dirty Flag )
	bool boolGlobalDirtyFlag = false;

	// Function to Retrieve Global Context Dirty Flag ( and Reset it if required )
	bool boolGlobalDirty( bool boolReset = false )
	{
		if ( !boolGlobalDirtyFlag )
			return( false );
			
		if ( boolReset )
			boolGlobalDirtyFlag = false;
			
		return( true );
	}

	// Function to Retrieve Global Context Variable, set it if non-existing
	double dblGlobalVariableRead( string strName,
		double dblValue = 0, string strPrefix = "" )
	{
		string strID = StringConcatenate( strPrefix, ".", strName );
		if ( boolParamResetFlag || !GlobalVariableCheck( strID ) )
			GlobalVariableSet( strID, dblValue );
			
		double dblValueNew = GlobalVariableGet( strID );
		
		if ( ( dblValueNew - dblValue ) != 0.0 )
			boolGlobalDirtyFlag = true;
		
		return( dblValueNew );
	}
	

//+------------------------------------------------------------------------------------+
//	Functions for Handling Logs and Comments
//+------------------------------------------------------------------------------------+

   // Function to Print to Log and/or Comment to Chart a String
	void voidCommentPrintString(
			string strText, bool boolComment = true, bool boolPrint = true )
	{
		if( IsOptimization() ) return;
		if( IsTesting() &&  !IsVisualMode() ) boolComment = false;
		
		if ( boolComment )	Comment( strText );
		if ( boolPrint )	   Print( strText );
	}

	// Function to Print to Log a String
	void voidPrint( string strText )
	{
		voidCommentPrintString( strText, false, true );
	}
	
	// Function to Print to Log and Comment to Chart a String
	void voidCommentPrint( string strText )
	{
		voidCommentPrintString( strText, true, true );
	}
	
	
//+------------------------------------------------------------------------------------+
//	Extra Support Variables and Functions
//+------------------------------------------------------------------------------------+

   // Functions to Calculate Price Spread
	double dblSpreadPrice( double dblAskPrice, double dblBidPrice )
	{	return( fabs( dblAskPrice - dblBidPrice ) );	}
	
	// Compare if two double values are Equal
	bool boolDblEqual( double dblValue1, double dblValue2, int intDigits = -1 )
	{
		if ( intDigits < 0 )
			intDigits = (int) MarketInfo( Symbol(), MODE_DIGITS );	// "Digits"
			
		return( NormalizeDouble( dblValue1 - dblValue2, intDigits )
		          == NormalizeDouble( 0, _Digits ) );
	}
	
	// Function for Random Sleep Time
   void voidSleepRandom( double dblTimeMean, double dblTimeMax )
   {
	   if( IsTesting() ) return;
	   
   	double dblTenths = ceil( dblTimeMean * 10 );
   	if ( dblTenths <= 0 ) return;

   	int intTenthsMax = (int) round( dblTimeMax * 10 );
   	double
   	   dblP     = 1.0 - 1.0 / (double) intTenthsMax
   	,  dblPrnd  = dblP * 32768
   	;

	   Sleep(10); // Always sleep at least one-hundredth second

	   // Now loop through and sleep 1/10th second a max of 
	   // (10 * mean_time) times.  But break out "randomly".
   	for( int i = 0; i < intTenthsMax; i++ )
   	{
   		if ( rand() > dblPrnd ) break;
   		Sleep(100);
   	}
   }

	
//+------------------------------------------------------------------------------------+